
#ifndef GALLNAV_H
#define GALLNAV_H
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: 
  Version: $ Rev: $

  2011-02-14 /JD: created

-*/

#include <iostream>
#include <string.h>

#include "gdata/gnav.h"
#include "gdata/geph.h"
#include "gdata/gdata.h"
#include "gdata/gnavgps.h"
#include "gdata/gnavglo.h"
#include "gutils/gconst.h"
#include "gutils/gtime.h"

#define MAX_GPS_PRN 32
#define MAX_GLO_PRN 24
#define MAX_GAL_PRN 30
#define NAV_BUF   1024


using namespace std;

class t_gallnav : public t_gdata {

 public:
  t_gallnav();
  virtual ~t_gallnav();

  typedef map<t_gtime, t_geph*>         t_map_ref;     // all data for a single satellite
  typedef map<string, t_map_ref>        t_map_sat;     // all data for all satellites
   
  virtual vector<string> satellites();                 // get all satellites
     
  virtual int pos( string sat, const t_gtime& t, double  xyz[3], double  var[3] = NULL, double  vel[3] = NULL ); // [m]
  virtual int clk( string sat, const t_gtime& t, double* clk,    double* var    = NULL, double* dclk   = NULL ); // [s]

  virtual void clean_outer( const t_gtime& beg = t_gtime::first_time(), 
			    const t_gtime& end = t_gtime::last_time() );

  virtual unsigned int nepochs( const string& prn );                 // get number of epochs
  virtual t_gtime beg_gnav( string prn );                            // get first t_gnav epoch
  virtual t_gtime end_gnav( string prn );                            // get last  t_gnav epoch
  virtual t_gtime beg_time( string prn ){ return beg_gnav(prn); }    // get first t_gephrtcm epoch
  virtual t_gtime end_time( string prn ){ return end_gnav(prn); }    // get first t_gephrtcm epoch
     
  int add( const t_gnav* nav );                        // add single navigation message

  void overwrite(bool b){ _overwrite = b; }            // set/get overwrite mode
  bool overwrite(){ return _overwrite; }

  void offset( int i ){ _offset = i; }                 // set/get offset for satdata
  int  offset(){ return _offset; }

 protected:

  virtual t_geph* _find( string sat, const t_gtime& t );// find appropriate t_geph element
  virtual t_geph* _find( string sat, int iod          );// find appropriate t_gnav element

  int               _offset;      // offset for RTCM corrections
  int               _nepoch;      // maximum number of epochs (0 = keep all)
  t_map_sat         _mapsat;      // map over all satellites (positions,..?)
  bool              _overwrite;   // rewrite/add only mode
  
 private:
};

#endif
