
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <stdlib.h>
#include <iostream>
#include <iomanip>
#include <sstream>

#include "gall/gallobs.h"
#include "gutils/gtypeconv.h"
#include "gutils/gfreq.h"


using namespace std;


// constructor
// ----------
t_gallobs::t_gallobs() 
  : t_gdata(),
    _nepoch(0),
    _overwrite(false)
{
}


// destructor
// ----------
t_gallobs::~t_gallobs()
{
  map<string,t_map_ref>::iterator itSITE = _mapobj.begin();
  while( itSITE != _mapobj.end() ){
    string site = itSITE->first;
     
//    cout << " site: " << site << endl;     cout.flush();

    map<t_gtime,t_map_sat>::iterator itEPO = _mapobj[site].begin();
    while( itEPO != _mapobj[site].end() ){
       
      t_gtime t = itEPO->first;
//      cout << " delete --- time: " << t.str("%Y-%m-%d %H:%M:%S[%T]: "); cout.flush();


      // JD
      //map<string,t_gobsgnss*>::iterator itSAT = _mapobj[site][t].begin(); 
      map<string,t_gobsgnss>::iterator itSAT = _mapobj[site][t].begin();
      while( itSAT != _mapobj[site][t].end() ){

// JD
//        t_gobsgnss* itDEL = itSAT->second;
        t_gobsgnss itDEL = itSAT->second;

//        cout << " " << itSAT->first; cout.flush();

//	cout << "destruuji: "
//	     << itDEL.sat()
//	     << itDEL.epoch().str("  %Y-%m-%d %H:%M:%S[%T] ")
//	     << fixed    << setprecision(3)
//	     << setw(16) << itDEL.getobs(L1)*0.19
//	     << setw(16) << itDEL.phase_lc(1,1,0,0)
//	     << setw(16) << itDEL.MW()
//           << setw(16) << itDEL.L4()
//	     << endl;
//	 vector<GOBS> v_obs = itDEL.obs();
//	 vector<GOBS>::iterator itOBS = v_obs.begin();
//	 for( ; itOBS != v_obs.end(); ++itOBS ) cout << " " << t_gobs::gobs2str( *itOBS ) << ":" << itDEL.getobs( *itOBS );
//	 cout  << endl; cout.flush();

	// JD
    //    delete itSAT->second;
	    itSAT++;   // or  _mapobj[site][t].erase(itSAT++);
      }
      itEPO++;     // or  _mapobj[site].erase(itEPO++);

      _mapobj[site][t].clear();
//      cout << endl; cout.flush();
    }
    itSITE++;     // or _mapobj.erase(itSITE++);
    _mapobj[site].clear();
  }
  _mapobj.clear();
}

// Observation statistics
// ---------------------------------
t_gallobs::t_map_obs t_gallobs::obsstat(string site)
{
   boost::mutex::scoped_lock lock(_mutex);
   
   t_map_obs obsStat;

   if( _mapobj.find(site) == _mapobj.end() ) return obsStat;
   
   // iterators
   t_map_obs::iterator itGSYS;
   map<string, map<GOBS, pair<int,int> > >::iterator itPRN;
   map<GOBS, pair<int,int> >::iterator itGOBS;
   map<t_gtime, t_map_sat>::const_iterator itEpo;
   map<string, t_gobsgnss>::iterator itSat;
   
   vector<GOBS>::iterator itVOBS;
   map<GSYS, map<string, int> > tot;
   t_gtime t;

   for( itEpo = _mapobj[site].begin(); itEpo != _mapobj[site].end(); itEpo++ ) // loop over epochs
   {
      t = itEpo->first;
      for( itSat = _mapobj[site][t].begin(); itSat != _mapobj[site][t].end(); itSat++ ) // loop over satellites
      {
	 GSYS   sys = itSat->second.gsys();
	 string sat = itSat->second.sat();
	 
	 tot[sys][sat] += 1;
	 vector<GOBS> obs = itSat->second.obs();
	 for( unsigned int i=0; i < obs.size(); i++ ){

	    if( itSat->second.getobs(obs[i]) != 0.0 ){
	      obsStat[sys][sat][obs[i]].first += 1;
	    }	    
	    obsStat[sys][sat][obs[i]].second = tot[sys][sat];
	 }
      }      
   }

   return obsStat; 
}

// return set of available GNSS systems
// --------------------------
set<GSYS> t_gallobs::sys(string site)
{
   map<t_gtime, t_map_sat>::const_iterator itEpo;
   map<string, t_gobsgnss>::iterator itSat;   
   set<GSYS> temp;
   t_gtime t;
   
   for (itEpo = _mapobj[site].begin(); itEpo != _mapobj[site].end(); itEpo++) // loop over epochs
   {
   t = itEpo->first;
     for(itSat = _mapobj[site][t].begin(); itSat != _mapobj[site][t].end(); itSat++) // loop over satellites
     {
	temp.insert(itSat->second.gsys());
     }
   }   
   return temp;
}


// return list of available stations
// ----------
vector<string> t_gallobs::stations()
{
  boost::mutex::scoped_lock lock(_mutex);

  vector<string> all_sites;
  map<string,t_map_ref>::const_iterator itSITE = _mapobj.begin();
   
  while( itSITE != _mapobj.end() ){
    all_sites.push_back(itSITE->first);
    itSITE++;
  }
  return all_sites;
}


// return list of available satellites
// ----------
/*
vector<t_gobsgnss> t_gallobs::obs(const string& site, const t_gtime& t)
{
  boost::mutex::scoped_lock lock(_mutex);
  vector<t_gobsgnss> all_obs;

  if( _mapobj.find(site)     == _mapobj.end() ||
      _mapobj[site].find(t)  == _mapobj[site].end() ) return all_obs;

  map<string,t_gobsgnss*>::const_iterator itSAT = _mapobj[site][t].begin();
  while( itSAT != _mapobj[site][t].end() ){
     
    t_gobsgnss gobs(*(itSAT->second));
    all_obs.push_back(gobs);
    itSAT++;
  }
  return all_obs;
}
*/


// return list of available satellites
// ----------
vector<string> t_gallobs::sats(const string& site, 
                               const t_gtime& t)
{
  boost::mutex::scoped_lock lock(_mutex);
  vector<string> all_sats;

  if( _mapobj.find(site)    == _mapobj.end() ||
      _mapobj[site].find(t) == _mapobj[site].end() ) return all_sats;

// JD
//  map<string,t_gobsgnss*>::const_iterator itSAT = _mapobj[site][t].begin();
  map<string,t_gobsgnss>::const_iterator itSAT = _mapobj[site][t].begin();  
  while( itSAT != _mapobj[site][t].end() ){
    all_sats.push_back(itSAT->first);
    itSAT++;
  }
  return all_sats;
}


// return list of available satellites
// ----------
vector<t_gsatdata> t_gallobs::obs(const string& site, 
                                  const t_gtime& t)
{
  boost::mutex::scoped_lock lock(_mutex);

  vector<t_gsatdata> all_obs;
  t_gtime tt(t_gtime::GPS);

  if( _mapobj.find(site) == _mapobj.end() ){
//      cout << "site not found: " << site << t.str_ymdhms() << endl; cout.flush();
     return all_obs;
  }
    
   
 //  _mapobj[site].find(t) // nelze aplikovat pokud neni presny cas!
     
  map<t_gtime,t_map_sat>::iterator it1 = _mapobj[site].lower_bound(t);  // greater || equal
  map<t_gtime,t_map_sat>::iterator it0 = it1;                           // previous value
//  map<t_gtime,t_map_sat>::iterator it  = it1;                           // the closest value

  if( it0 != _mapobj[site].begin() ) it0--;                             // set previous value  
  if( it1 == _mapobj[site].end() ) it1 = it0;

  if( it1 == _mapobj[site].end() && it0 == _mapobj[site].end() ){
    cout << site << " " << t.str_ymdhms() << " " << t.dsec() << " no observations !\n";
    return all_obs;
  }
     
#ifdef DEBUG
    cout << site << " " << t.str_ymdhms() << " " << t.dsec() << fixed << setprecision(6)
         << " t1_diff: " << setw(14) << it1->first.diff(t)
         << " t0_diff: " << setw(14) << it0->first.diff(t)
         << endl;
#endif

  if(      fabs(it1->first - t) <= DIFF_SEC ) tt = it1->first;  // it = it1;                 // set closest value
  else if( fabs(it0->first - t) <= DIFF_SEC ) tt = it0->first;  // it = it0;                 // set closest value
  else{
#ifdef DEBUG
     if( _log ){
       ostringstream lg;
       lg << site << tt.str_ymd()  << tt.dsec()
          << " it0 = " << it0->first.str_ymdhms() << " diff: " << it1->first.diff(t)
          << " it1 = " << it1->first.str_ymdhms() << " diff: " << it0->first.diff(t)
          << " not close observations found!";
       _log->comment(0,"gallobs",lg.str());
     }     
#endif
     return all_obs;                                                    // not found !     
  }
   
// JD
// map<string,t_gobsgnss*>::const_iterator itSAT = _mapobj[site][tt].begin();
  map<string,t_gobsgnss>::iterator itSAT = _mapobj[site][tt].begin();
  while( itSAT != _mapobj[site][tt].end() ){
     
    string sat = itSAT->first;
/////    t_gsatdata* gobs = new t_gsatdata( *(itSAT->second) );
    // JD
    //all_obs.push_back( *(itSAT->second) );    
    all_obs.push_back( itSAT->second );
 
    itSAT++;
  }

  return all_obs;
}


// return list of available satellites
// ----------
vector<t_gtime> t_gallobs::epochs(const string& site)
{
  boost::mutex::scoped_lock lock(_mutex);
  vector<t_gtime> all_epochs;

  if( _mapobj.find(site) == _mapobj.end() ){
    // cout << "site not found: " << site << t.str_ymdhms() << endl; cout.flush();
     return all_epochs;
  }
     
  map<t_gtime,t_map_sat>::iterator it = _mapobj[site].begin();

  while( it != _mapobj[site].end() ){
    all_epochs.push_back( it->first );
    it++;
  }
  return all_epochs;
}


// return first position for satellite
// ----------
t_gtime t_gallobs::beg_obs(const string& site)
{
  boost::mutex::scoped_lock lock(_mutex);

  if( _mapobj.find(site)    != _mapobj.end() &&
      _mapobj[site].begin() != _mapobj[site].end() )
    return _mapobj[site].begin()->first;

  return t_gtime::last_time();
}


// return last position for satellite
// ----------
t_gtime t_gallobs::end_obs(const string& site)
{
  boost::mutex::scoped_lock lock(_mutex);

  if( _mapobj.find(site)    != _mapobj.end() &&
      _mapobj[site].begin() != _mapobj[site].end() )
    return _mapobj[site].rbegin()->first;

        

  return t_gtime::first_time();
}


// add observations ( both P and L in meters !!!)
// ----------
// JD
//int t_gallobs::addobs(t_gobsgnss* obs)
int t_gallobs::addobs(const t_gobsgnss& obs)
{
  t_gtime tt(obs.epoch());
  string site = obs.site();
  string sat  = obs.sat();

  boost::mutex::scoped_lock lock(_mutex);

  t_map_ref::const_iterator itEPO = _mapobj[site].find(tt);
  t_map_sat::const_iterator itSAT = _mapobj[site][tt].find(sat);

  // add new observations (or overwrite)
  // ===================================
  if( _overwrite || itEPO == _mapobj[site].end()        
                 || itSAT == _mapobj[site][tt].end()
  ){
#ifdef DEBUG
    cout << " ADDING " << site << " " <<  _mapobj[site].size() << " " << _nepoch << " " <<  nepochs(site)
         << " sat "    <<  sat << " " << tt.str_ymdhms() << " " << "\n";cout.flush();
#endif

    // delete old if exists
    // ====================
    if( _overwrite && itEPO != _mapobj[site].end()
                   && itSAT != _mapobj[site][tt].end()
    ){
// JD    //delete _mapobj[site][tt][sat];
        _mapobj[site][tt].erase(sat);
        if( _log && _log->verb() >= 2 ) _log->comment(2, "gallobs", site + tt.str_ymdhms(" obs replaced ") + sat);
    }

    // too many epochs (remove old)
    // ============================
    if( _nepoch > 0 && _mapobj[site].size() > _nepoch + 10 ){  // +10 .. reduce number of removals to every tenths

      map<t_gtime,t_map_sat>::iterator itEPO = _mapobj[site].begin();
      while( itEPO != _mapobj[site].end() ){

        if( _mapobj[site].size() <= _nepoch ){ break; }
	
        t_gtime t = itEPO->first;
        if( _log && _log->verb() >= 2 )
	{	 	      
           string satells(" ");
// JD      map<string,t_gobsgnss*>::iterator itSAT;
           map<string,t_gobsgnss>::iterator itSAT;
           for( itSAT = _mapobj[site][t].begin(); itSAT != _mapobj[site][t].end(); ++itSAT ){
             satells = satells + " " + itSAT->first;
// JD        delete itSAT->second; // do later all at once, otherwise need to do while + erase(itSat++);
           }
       
           _log->comment(2, "gallobs", site + t.str_ymdhms(" obs removed ") + satells);
	}

        _mapobj[site][t].erase(_mapobj[site][t].begin(),_mapobj[site][t].end());
        _mapobj[site].erase(itEPO++);
      }
    }

    // distinguish GNSS specific message
    // =================================
    if( obs.id_type() == t_gdata::OBSGNSS ){
// JD   _mapobj[site][tt][sat] = new t_gobsgnss( obs );  // allocate in heap
// NEBO       _mapobj[site][tt][sat] = obs; // jiz pointer

      _mapobj[site][tt][sat] = t_gobsgnss( obs );
      //_mapobj[site][tt].insert(pair<string,t_gobsgnss>( sat, obs )); // totez co se tyce rychlosti!!!

    }else{
       if( _log ){  _log->comment(0,"gallobs","warning: t_gobsgnss record not identified!"); }
       return 1;
    }
  }else{
    if( _log && _log->verb() >= 2 ) _log->comment(2,"gallobs", site + tt.str_ymdhms(" skipped ") + " " + sat);
    return 0;
  }

  // comments
  if( _log && _log->verb() >= 3 ){

    t_gobsgnss  tmp( obs );
    vector<GOBS> v_obs = tmp.obs();
    vector<GOBS>::iterator itOBS = v_obs.begin();
    ostringstream lg; lg << fixed << setprecision(3);
    for( ; itOBS != v_obs.end(); ++itOBS ) 
      lg << " " << t_gobs::gobs2str( *itOBS ) << ":" << tmp.getobs( *itOBS );
  
    _log->comment(3,"gallobs",site + tt.str_ymdhms(" add obs: ") + " " + sat + lg.str() );
  }
  return 0;
}


// clean function
// ----------
void t_gallobs::clean_outer( const  string& obj,
                             const t_gtime& beg, 
                             const t_gtime& end )
{
  if( end < beg ) return;
  boost::mutex::scoped_lock lock(_mutex);
   
  // loop over all object [default]
  map<string,t_map_ref>::const_iterator itOBJ  = _mapobj.begin();
  map<string,t_map_ref>::const_iterator itOBJ2 = _mapobj.end();
    
  if( _log && _log->verb() >= 2 ){
    _log->comment(2,"gallobs","obs clean request: " + obj + beg.str_ymdhms(": ") + end.str_ymdhms(" - "));
  }
     
  // loop over a single object (if obj defined !)
  if( ! obj.empty() ) itOBJ = itOBJ2 = _mapobj.find(obj);
  if( itOBJ2 != _mapobj.end() ) itOBJ2++;

  for( ; itOBJ != itOBJ2; ++itOBJ ){

    string site = itOBJ->first;

    // find and CLEAN all data (epochs) out of the specified period !
    map<t_gtime,t_map_sat>::iterator it;
    map<t_gtime,t_map_sat>::iterator itFirst = _mapobj[site].begin();
    map<t_gtime,t_map_sat>::iterator itLast  = _mapobj[site].end();
    map<t_gtime,t_map_sat>::iterator itBeg   = _mapobj[site].lower_bound(beg);  // greater|equal
    map<t_gtime,t_map_sat>::iterator itEnd   = _mapobj[site].upper_bound(end);  // greater only!

//  JD
//  map<string,t_gobsgnss*>::iterator itOBS;
    map<string,t_gobsgnss>::iterator itOBS;

    // delete before BEGIN request
    if( itBeg != itFirst ){
      if( _log && _log->verb() >= 3  ){
        string lg("eph removed before: " + itBeg->first.str_ymdhms() + " " + site);
        _log->comment(3,"gallobs",lg);
      }

      for( it = itFirst; it != itBeg; ++it ){	 
        t_gtime tt(it->first);
// JD
//	for( itOBS  = _mapobj[site][tt].begin();
//         itOBS != _mapobj[site][tt].end(); 
//         ++itOBS ) delete itOBS->second;
	 
        _mapobj[site][tt].erase(_mapobj[site][tt].begin(),_mapobj[site][tt].end());
      }
       
      _mapobj[site].erase(itFirst,itBeg); // erase all but itBeg
    }

    // delete after END request
    if( itEnd != itLast ){ // && ++itEnd != itLast ){
      if( _log && _log->verb() >= 3 ){
        string lg("eph removed after : " + itEnd->first.str_ymdhms() + " " + site);
        _log->comment(3,"gallobs",lg);
      }

      for( it = itEnd; it != itLast; ++it ){
	    
        t_gtime tt(it->first);
//JD
//	for( itOBS  = _mapobj[site][tt].begin();
//	     itOBS != _mapobj[site][tt].end(); 
//	     ++itOBS ) delete itOBS->second;
	 
        _mapobj[site][tt].erase(_mapobj[site][tt].begin(),_mapobj[site][tt].end());	 
      }                   
      _mapobj[site].erase(itEnd,itLast);
    }
//    itOBJ++; // WHILE LOOP ONLY
  }
}


// return list of available satellites
// ----------
map<string,t_gobsgnss>* t_gallobs::find( string site, const t_gtime& t)
{
  boost::mutex::scoped_lock lock(_mutex);
  if( _mapobj.find(site)         == _mapobj.end() ||
      _mapobj[site].find(t)      == _mapobj[site].end() ) return 0;

  return  &_mapobj[site][t];
}
// smart pointer version
/* unique_ptr<map<string,t_gobsgnss> > t_gallobs::find( string site, const t_gtime& t) */
/* { */
/*      if( _mapobj.find(site)         == _mapobj.end() || */
/*       _mapobj[site].find(t)      == _mapobj[site].end() ) return 0; */

/*    unique_ptr<map<string,t_gobsgnss> > smart(&_mapobj[site][t]); */
/*    return smart; */
/* } */


// number of epochs for station
// ----------
unsigned int t_gallobs::nepochs(const string& site)
{
  boost::mutex::scoped_lock lock(_mutex);

  if( _mapobj.find(site) == _mapobj.end() ) return 0;

/*
  if( site == "VSBO0" ){     
    map<t_gtime,t_map_sat>::iterator itEPO = _mapobj[site].begin();
    while( itEPO != _mapobj[site].end() ){
       
      t_gtime t = itEPO->first;
      cout << site << " --- time: " << t.str_ymdhms() << " " << t.dsec() << endl;
      itEPO++;
    }
  }
*/

  return _mapobj[site].size();
}

unsigned int t_gallobs::nepochs(const string& site, const t_gtime& beg, const t_gtime& end, const double& sampl, pair<int,int>& n)
{
   if( _mapobj.find(site) == _mapobj.end() ) return 0;
   boost::mutex::scoped_lock lock(_mutex);
   
   n.first = n.second = 0;
   
   t_gtime tt = beg;
   while (tt < end || tt == end)
   {
      // JE TREBA UPRAVIT!!!! MUTEX
      lock.unlock();
      vector<t_gsatdata> satdata = obs(site, tt);
      lock.lock();
      if (satdata.size() > 0) n.second += 1;
      tt = tt + sampl;
   }   

   n.first = floor( (end - beg) / sampl + 0.5) + 1;
   
   return 1;
}

// gaps number
// ---------------------
int t_gallobs::gaps(string site, int gap, map<t_gtime, int>& len)
{
   if( _mapobj.find(site) == _mapobj.end() ) return 0;
   boost::mutex::scoped_lock lock(_mutex);
   
   map<t_gtime, t_map_sat>::const_iterator itEpo;
   t_gtime t;
   t_gtime t_sav = _mapobj[site].begin()->first;
   
   for (itEpo = _mapobj[site].begin(); itEpo != _mapobj[site].end(); itEpo++) // loop over epochs
   {
      t = itEpo->first;
      if (abs(t-t_sav) > gap){
        len[t_sav] = abs(t-t_sav);
      }
      
      t_sav = t;
   }
   
   return 1;
}
 
// short block of data
// ----------------------------
int t_gallobs::block(string site, int b, int g, map<t_gtime, int>& len)
{
   if( _mapobj.find(site) == _mapobj.end() ) return 0;
   boost::mutex::scoped_lock lock(_mutex);
   
   map<t_gtime, t_map_sat>::const_iterator itEpo;
   t_gtime t;
   t_gtime t_sav = _mapobj[site].begin()->first;
   t_gtime sb = _mapobj[site].begin()->first;
   t_gtime eb;
   
   for (itEpo = _mapobj[site].begin(); itEpo != _mapobj[site].end(); itEpo++) // loop over epochs
   {
      t = itEpo->first;
      if (abs(t-t_sav) > g){
	 eb = t_sav;      
        if (abs(eb-sb) < b){
	   len[sb] = abs(eb-sb);
	   sb = t;
        }
	else sb = t;
      }
            
      t_sav = t;
   }
   
   return 1;
}

// number of code/phase bands
// ----------------------------------------
int t_gallobs::nbands(string site, map<GSYS, map<t_gtime, map<string, pair<int,int> > > >& nb, int sampl)
{
   if (_mapobj.find(site) == _mapobj.end()) return -1;
   boost::mutex::scoped_lock lock(_mutex);
   
   map<t_gtime, t_map_sat>::const_iterator itEpoB = _mapobj[site].begin();
   map<t_gtime, t_map_sat>::const_iterator itEpoE;
   map<string,t_gobsgnss>::iterator itSat;   
   t_gtime t;

   // all epochs are involved
   if (sampl == 0){
     while (itEpoB != _mapobj[site].end())
     {
	t = itEpoB->first;
        for(itSat = _mapobj[site][t].begin(); itSat != _mapobj[site][t].end(); itSat++) // loop over satellites
        {
	  GSYS sys = itSat->second.gsys();
          string prn = itSat->first;
          itSat->second.nbands(nb[sys][t][prn]);
        }	
	itEpoB++;
     }
     return 1; 
   }
   
   // just sampled epochs are involved
   itEpoB = _mapobj[site].begin();
   itEpoE = _mapobj[site].end(); itEpoE--;
   t = itEpoB->first;
   
   while (t < itEpoE->first || t == itEpoE->first)
   {
     for(itSat = _mapobj[site][t].begin(); itSat != _mapobj[site][t].end(); itSat++) // loop over satellites
     {
	GSYS sys = itSat->second.gsys();	
        string prn = itSat->first;
        itSat->second.nbands(nb[sys][t][prn]);
     }
     t = t+sampl; 
   }  
   
   return 1;
}

/*
// get data from one station over all epochs
// -----------------------------------------------
//
//  ????? TO discussion: return value or pointer? ?????????
// 
// map<t_gtime, map<string, t_gobsgnss*> > t_gallobs::obsSiteAll(string site)
t_gallobs::t_map_ref* t_gallobs::obsSiteAll(const string& site)
{
   return &_mapobj[site];
}
*/

// return true if map contains the site and false if not
//-----------------------------------------------------
bool t_gallobs::isSite(string site)
{
   boost::mutex::scoped_lock lock(_mutex);
   if ( _mapobj.find(site) == _mapobj.end() ) return false;
   else return true;
}
