
#ifndef GALLOBS_H
#define GALLOBS_H
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: observation container
  Version: $ Rev: $

  2012-05-02 /JD: created

-*/

#include <iostream>
#include <string.h>
#include <set>
#include <memory>

#include "gdata/gdata.h"
#include "gdata/gobsgnss.h"
#include "gdata/gsatdata.h"
#include "gutils/gconst.h"
#include "gutils/gtime.h"

#define DIFF_SEC 0.905  // [sec] returns observations within +- DIFF_SEC

using namespace std;

class t_gallobs : public t_gdata {

 public:
  t_gallobs();
  virtual ~t_gallobs();

// JD
//  typedef map<string, t_gobsgnss*>       t_map_sat;      // all data-types/single epoch
  typedef map<string, t_gobsgnss>        t_map_sat;      // all data-types/single epoch
  typedef map<t_gtime, t_map_sat>        t_map_ref;      // all data-types/all epochs/single object
  typedef map<string, t_map_ref>         t_map_obj;      // all data-types/all epochs/all objects
   
  typedef map<GSYS, map<string, map<GOBS, pair<int,int> > > > t_map_obs;
  virtual vector<string> stations();                     // get all stations

  virtual map<GSYS, map<string, map<GOBS, pair<int,int> > > > obsstat(string site);    // (PV)
  virtual set<GSYS> sys(string site);
          int gaps(string site, int g, map<t_gtime,int>& l);  // gaps number
          int block(string site, int b, int g, map<t_gtime,int>& l);  // short block data
          int nbands(string site, map<GSYS, map<t_gtime, map<string, pair<int,int> > > >& nb, int sampl = 0); // phase/code band number
  virtual bool isSite(string site);
   
  virtual vector<string> sats(const string& site, 
                              const t_gtime& t);         // get all satellites for epoch t
//virtual vector<t_gobsgnss> obs(const string& site, const t_gtime& t);// get all t_gobsgnss for epoch t
  virtual vector<t_gsatdata> obs(const string& site,
                                 const t_gtime& t);      // get all t_gsatdata for epoch t
  virtual vector<t_gtime>    epochs(const string& site); // get all t_gepochs for site
   
  virtual void clean_outer(const string& site = "",
                           const t_gtime& beg = t_gtime::first_time(),
			               const t_gtime& end = t_gtime::last_time());

  virtual t_gtime beg_obs(const string& site);           // get first t_gnav epoch for site
  virtual t_gtime end_obs(const string& site);           // get last  t_gnav epoch for site
   
// JD
//  int addobs(t_gobsgnss* obs);                     // add single station observation (P and L in meters !)
  int addobs(const t_gobsgnss& obs);                     // add single station observation (P and L in meters !)
   
  void overwrite(bool b){ _overwrite = b; }              // set/get overwrite mode
  bool overwrite(){ return _overwrite; } 

  void maxepoch( unsigned int i ){ _nepoch = i; }        // set/get maximum number of epochs stored
  unsigned int maxepoch(){ return _nepoch; }

  unsigned int nepochs(const string& site);              // get number of epochs for station
  unsigned int nepochs(const string& site, const t_gtime& beg, const t_gtime& end,  const double& sampl, pair<int,int>& n);              // get number of epochs for station expect/have according to sampl

//  t_map_ref* obsSiteAll(const string& site);             // get map of one station over all epochs

  virtual t_map_sat* find( string site, const t_gtime& t);// find appropriate t_gobsgnss element for site/epoch
//  virtual unique_ptr< map<string,t_gobsgnss> > find( string site, const t_gtime& t);// find appropriate t_gobsgnss element for site/epoch -> smart pointer version
   
 protected:

  unsigned int      _nepoch;      // maximum number of epochs (0 = keep all)
  t_map_obj         _mapobj;      // map over all objects (receivers)
  bool              _overwrite;   // rewrite/add only mode
    
 private:
};


#endif
