
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include "gall/gallotl.h"

//Constructor
t_gallotl::t_gallotl()
{
}

//Destructor
t_gallotl::~t_gallotl()
{
   _mapotl.clear();
}

int t_gallotl::data(Matrix& otldata, const string& site)
{
  boost::mutex::scoped_lock lock(_mutex);
   
   if (_mapotl.find(site) == _mapotl.end() || _mapotl.size() == 0)
     return -1;
   else otldata = _mapotl[site].data();

   return 1;
}

double t_gallotl::lat(const string& site)
{
  boost::mutex::scoped_lock lock(_mutex);
   
   if (_mapotl.find(site) == _mapotl.end())
     return 0;
   else return _mapotl[site].lat();
}

double t_gallotl::lon(const string& site)
{
  boost::mutex::scoped_lock lock(_mutex);
   
   if (_mapotl.find(site) == _mapotl.end())
     return 0;
   else return _mapotl[site].lon();
}


// Add element to map container
// -----------------------------
void t_gallotl::add(t_gotl& otl)
{
   boost::mutex::scoped_lock lock(_mutex);
   
   string site = otl.site();

   _mapotl[site] = otl;   
}

// Print all data
// --------------------
void t_gallotl::print()
{
   map<string, t_gotl>::iterator it;
   for (it = _mapotl.begin(); it != _mapotl.end(); it++)
   {
      t_gotl otl = it->second;
      cout << "Site: " << otl.site() << " Lon: " << otl.lon() << " Lat: " << otl.lat() << endl;
      cout << "Data: \n" << otl.data() << endl;
   }
   
}
