
#ifndef GCODER_H
#define GCODER_H 
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: 
  Version: $Rev:$

  2011-04-20 /JD: created
  2013-03-07 /JD: general settings supported

-*/

#include <set>
#include <vector>
#include <string>
#include <cstring>
#include <boost/thread/mutex.hpp>

#include "gdata/gdata.h"
#include "gset/gsetgen.h"

#define BUFFER_INCREASE_FAC  1.5
#define DEFAULT_BUFFER_SIZE  4096
#define MAXIMUM_BUFFER_SIZE  9000000

// enum t_irc { failure = -1, success, fatal } ; // return code

static const char lf = 0x0a; // = "\n" .. line feed
static const char cr = 0x0d; // = "\r" .. carriage return

#ifdef __linux__
static const string crlf = string("") + lf;   // "\n"
#endif

#if defined _WIN32 || defined _WIN64
static const string crlf = string("") + cr + lf; // "\r\n"
#endif 

using namespace std;

class t_gcoder {

 public:
   t_gcoder( string version = "", int sz = DEFAULT_BUFFER_SIZE );
   virtual ~t_gcoder();

   virtual void clear();

   virtual void version(string s){ _version = s; }
   virtual string version(){ return _version; }

   virtual int decode_head( char* buff, int sz,           vector<string>& errmsg ){ return -1; } // = 0;
   virtual int decode_data( char* buff, int sz, int& cnt, vector<string>& errmsg ){ return -1; } // = 0;

   virtual int encode_head( char* buff, int sz,           vector<string>& errmsg ){ return -1; } // = 0;
   virtual int encode_data( char* buff, int sz, int& cnt, vector<string>& errmsg ){ return -1; } // = 0;

   int add_data( string data_id, t_gdata* data );
   int rem_data( string data_id);
   t_gdata* data( string data_id ){ return _data[data_id]; }

   void gset(t_gsetgen& gen);                    // get settings

   void glog(t_glog* l){ _log = l; }             // set/get glog pointer
   t_glog* glog(){ return _log; }
   
 protected:
   int   endpos(){ return _endpos; }
   int     size(){ return _endpos; }             // number of char elements in buffer (excl \0)
   char* buffer(){ return _buffer; }

   virtual void _init();
   int _getline(string& str, int from_pos = 0);
   int _add2buffer( char* buff, int sz );
   int _consume( int bytes_to_eat );
   
   string                  _version;             // format version
   string                  _initver;             // format initial version
   map<string,t_gdata*>    _data;                // data pointer
   t_glog*                 _log;                 // log pointer
   t_gsetgen*              _set;                 // set pointer
   char*                   _buffer;              // class buffer
   int                     _buffsz;              // size of buffer
   int                     _endpos;              // after last position
   
   // gsetgen settings
   t_gtime _beg;                                 // default beg time
   t_gtime _end;                                 // default end time
   set<string> _sys;                             // default systems
   set<string> _rec;                             // default sites/receivers
   set<string> _sat;                             // default satellites
   set<string> _obs;                             // default observations
   
   boost::mutex            _mutex;               // buffer mutex
   
 private:

};

#endif
