
#ifndef GEPH_H
#define GEPH_H 
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements ephemerides and navigation classes
  Version: $ Rev: $

  2011-01-10 /JD: created

  TODO: overwritting mode
        return first/last
        limit for number of data inclusions per satellites
-*/

#include "gdata/gdata.h"
#include "gutils/gtime.h"
//#include "gutils/gconst.h"

//#define EPH_NAV   1    // bit 0
//#define EPH_RTCM  2    // bit 1
//#define EPH_SP3   4    // bit 2
//#define EPH_CLK   8    // bit 3

using namespace std;
    
class t_geph : public t_gdata {

 public:
  t_geph();
  virtual ~t_geph();
  
  // pointers to support NULL if not requested!
  virtual int pos( const t_gtime& t, double  xyz[3], double var[3] = NULL, double vel[3] = NULL ){ return -1; }  // [m]
  virtual int clk( const t_gtime& t, double*    clk, double*   var = NULL, double*  dclk = NULL ){ return -1; }  // [s]
  virtual int chk()    const { return -1; }
  virtual string str() const { return ""; }
  virtual int print()  const { cout << str(); return 0; }
   
  void clear();
  bool valid();

// SMART POINTERS!!! 
// http://en.wikipedia.org/wiki/Smart_pointer
// http://ootips.org/yonat/4dev/smart-pointers.html
// http://www.codeproject.com/Articles/15351/Implementing-a-simple-smart-pointer-in-c
// STL: auto_pt !!!
  t_geph* pointer(){ _npointers++; return this; }
  void    recount(){ _npointers--; }
//        release() // !!!      
  int     npointers( ){ return _npointers; }
   
  string  sat()      const{ return _sat; }                 // satellite number
  double  interval() const{ return _interval; }            // get validity interval
  t_gtime epoch()    const{ return _epoch; }               // reference epoch
  t_gtime begin()    const{ return _epoch - _interval/2; } // beg of validity
  t_gtime end()      const{ return _epoch + _interval/2; } // end of validity
   
 protected:
  virtual void _clear();
  virtual bool _valid() const;

  string            _sat;         // satellite number
  t_gtime           _epoch;       // reference epoch
  double            _interval;    // validity interval
  int               _npointers;   // counting given pointers
   
 private:
//  unsigned int      _types;

};

#endif
