
#ifndef  GOBSGNSS_H
#define  GOBSGNSS_H

/* ----------------------------------------------------------------------
   (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
       Research Institute of Geodesy, Topography and Cartography
       Ondrejov 244, 251 65, Czech Republic
   
   Purpose: implementation of GNSS observation element
   Version: $Rev:$
       
   2011-09-04 /JD: created
   2012-09-24 /JD: selections of code, phase etc.
   
   Todo: multi-GNSS & various LCs !
         gobj implementation

-*/

#include <string>
#include <vector>
#include "gdata/gdata.h"
#include "gutils/gtime.h"
#include "gutils/gobs.h"
#include "gutils/gfreq.h"

// priority tables for choice of available signals (code [m])
const static GOBS code_choise[9][19] = {
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   { C1A, C1B, C1C,   X,   X, C1L, C1M,   X, C1P,   X, C1S, C1W, C1X, C1Y, C1Z,  P1,  C1,  CA,  CB }, //  C1
   {   X,   X, C2C, C2D, C2I, C2L, C2M,   X, C2P, C2Q, C2S, C2W, C2X, C2Y,   X,  P2,  C2,  CC,  CD }, //  C2
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   {   X,   X,   X,   X, C5I,   X,   X,   X,   X, C5Q,   X,   X, C5X,   X,   X,  P5,  C5,   X,   X }, //  C5
   { C6A, C6B, C6C,   X, C6I,   X,   X,   X,   X, C6Q,   X,   X, C6X,   X, C6Z,   X,  C6,   X,   X }, //  C6
   {   X,   X,   X,   X, C7I,   X,   X,   X,   X, C7Q,   X,   X, C7X,   X,   X,   X,  C7,   X,   X }, //  C7
   {   X,   X,   X,   X, C8I,   X,   X,   X,   X, C8Q,   X,   X, C8X,   X,   X,   X,  C8,   X,   X }  //  C8
};

// priority tables for choice of available signals (phase [full-cycles])
const static GOBS phase_choise[9][19] = {
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   { L1A, L1B, L1C,   X,   X, L1L, L1M, L1N, L1P,   X, L1S, L1W, L1X, L1Y, L1Z,   X,  L1,  LA,  LB }, //  L1
   {   X,   X, L2C, L2D, L2I, L2L, L2M, L2N, L2P, L2Q, L2S, L2W, L2X, L2Y,   X,   X,  L2,  LC,  LD }, //  L2
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   {   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X,   X }, //
   {   X,   X,   X,   X, L5I,   X,   X,   X,   X, L5Q,   X,   X, L5X,   X,   X,   X,  L5,   X,   X }, //  L5
   { L6A, L6B, L6C,   X, L6I,   X,   X,   X,   X, L6Q,   X,   X, L6X,   X, L6Z,   X,  L6,   X,   X }, //  L6
   {   X,   X,   X,   X, L7I,   X,   X,   X,   X, L7Q,   X,   X, L7X,   X,   X,   X,  L7,   X,   X }, //  L7
   {   X,   X,   X,   X, L8I,   X,   X,   X,   X, L8Q,   X,   X, L8X,   X,   X,   X,  L8,   X,   X }  //  L8
};

using namespace std;

#define UNDEFINED_GOBS 0.000;

class t_gobsgnss : public t_gdata {
   
 public:

   t_gobsgnss() {};
//   t_gobsgnss(const t_gobsgnss& rf);
   t_gobsgnss(const string& site, const string& sat, const t_gtime& t);
   virtual ~t_gobsgnss();
   void addobs(const GOBS& obs,const double& d); // add a single observation
   void addlli(const GOBS& obs,const int& i);    // add a lost-of-lock indicator

   double    getobs(const GOBS&   obs);          // get a single observation
   double    getobs(const string& obs);          // get a single observation
   int       getlli(const GOBS&   obs);          // get a lost-of-lock indicator
   int       getlli(const string& obs);          // get a lost-of-lock indicator
   void      nbands(pair<int,int>& nb);          // get number of code/phase available bands
   
   size_t size() const;                     // return # of observations
   
// static string freq( FRQ freq );          // get string from FRQ enum
// static FRQ    freq( string freq );       // get FRQ enum from string
// static string obs( GOBS obs );           // get string from GOBS enum
// static GOBS   obs( string obs );         // get GOBS enum from string
   vector<GOBS> obs();                      // get vector of available observations
   
   GSYS gsys();                             // get GNSS system from satellite IDOC
   
   // duplicite with two other methods bellow (OBS-TYPE if pointer given)
   GOBS  code(const int& band,const GOBS& obs=X); // return  code ID of selected band, potentially preffered obs ID
   GOBS phase(const int& band,const GOBS& obs=X); // return phase ID of selected band, potentially preffered obs ID

   double obs_C(const int& band,GOBS* obs=0); //  code observation [m] for band i (1,2,5,6,7,8)
   double obs_L(const int& band,GOBS* obs=0); // phase observation [m] for band i (1,2,5,6,7,8)

   double  code_lc(const    int& band1,  // return value of general  code linear combination (c1*O1 + ... )
                   const double& coef1,  // for 2 or 3 bands with given coefficients
		           const    int& band2, 
                   const double& coef2,
		           const    int& band3 = 0, 
                   const double& coef3 = 0);

   double phase_lc(const    int& band1,  // return value of general phase linear combination (c1*O1 + ... ) 
                   const double& coef1,  // for 2 or 3 bands with given coefficients
		           const    int& band2, 
                   const double& coef2,
	 	           const    int& band3 = 0, 
                   const double& coef3 = 0);

   double  code_lcf(const int& band1,   // return value of general  code linear combination (c1*f1*O1 + ... / c1*f1 + ) 
                    const int& coef1,   // for 2 or 3 bands with given coefficients
	         	    const int& band2, 
                    const int& coef2,
	 	            const int& band3 = 0,
                    const int& coef3 = 0);
   
   double phase_lcf(const int& band1,   // return value of general phase linear combination (c1*f1*O1 + ... / c1*f1 + )
                    const int& coef1,   // for 2 or 3 bands with given coefficients
	 	            const int& band2, 
                    const int& coef2,   
		            const int& band3 = 0, 
                    const int& coef3 = 0);

// ------------
// temporary ??   
//   int ionofree_coef(int b1, int& c1,        // return coefficients (c1,c2) of the ionosphere-free linear combination
//                     int b2, int& c2);       // (2 bands)
//
//   int geomfree_coef(int b1, int& c1,        // return coefficients (c1,c2) of the geometry-free linear combination
//                     int b2, int& c2);       // (2 bands)
// temporary ??   
// ------------

   int ionofree_coef(const int& b1, double& c1,   // return coefficients (c1,c2) of the ionosphere-free linear combination
                     const int& b2, double& c2);  // (2 bands)
  
   int geomfree_coef(const int& b1, double& c1,   // return coefficients (c1,c2) of the geometry-free linear combination
                     const int& b2, double& c2);  // (2 bands)

   double P3(const int& b1=1, const int& b2=2);   // get ionosphere-free combination for code  [m]
   double L3(const int& b1=1, const int& b2=2);   // get ionosphere-free combination for phase [m]!
   double L4(const int& b1=1, const int& b2=2);   // get geometry-free combination for phase   [m]!
   double MW(const int& b1=1, const int& b2=2);   // get Melbourne-Wuebenna combination for phase & code [m]!

   t_gtime epoch() const{ return _epoch; }              // get reference epoch
   string  site()  const{ return _staid; }              // get station id
   string  sat()   const{ return _satid; }              // get satellite id
   string  sys()   const{ return _satid.substr(0,1); }  // get satellite system id
    
//   friend ostream &operator<<(ostream &stream, GOBS  obs);
//   friend istream &operator>>(istream &stream, GOBS &obs);

   void clear();
   bool valid();

 protected:
   virtual void _clear();
   virtual bool _valid() const;

   map<GOBS, double>    _gobs;  // maps of observations
   map<GOBS, int>       _glli;  // maps of lost-of-lock identifications
   
   string              _staid;  // station id
   string              _satid;  // satellite id ["G??", "R??", "E??" ...]
   t_gtime             _epoch;  // epoch of the observation

 private:
   
};

#endif
