
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <iostream>
#include <iomanip>
#include <sstream>

#include "gdata/gsatdata.h"

using namespace std;

// -----------
t_gsatdata::t_gsatdata(string site, string sat, const t_gtime& t)
  : t_gobsgnss(site,sat,t),
    _satcrd(0.0,0.0,0.0),
    _clk(0.0),
    _ele(0.0),
    _azi(0.0),
    _rho(0.0)
{ 
  id_type(t_gdata::SATDATA);
  id_group(t_gdata::GRP_OBSERV);
}


// -----------
t_gsatdata::t_gsatdata(const t_gobsgnss& obs)
  : t_gobsgnss(obs), // t_gobsgnss(obs.site() ,obs.sat(), obs.epoch() ),
    _satcrd(0.0,0.0,0.0),
    _clk(0.0),
    _ele(0.0),
    _azi(0.0),
    _rho(0.0)
{ 
//  _staid = obs.site();
//  _satid = obs.sat();
//  _epoch = obs.epoch();

//  _satcrd.resize(3);
//  _clk = 0.0;
//  _ele = 0.0;
//  _azi = 0.0;
//  _rho = 0.0;
       
  id_type(t_gdata::SATDATA);
  id_group(t_gdata::GRP_OBSERV);
}


// -----------
t_gsatdata::~t_gsatdata()
{
#ifdef DEBUG
  cout << "GSATDATA - destruct POCAT: " << site() << " " << sat() << " "
       << epoch().str("  %Y-%m-%d %H:%M:%S[%T] ") << fixed << setprecision(3) << endl;
   
  vector<GOBS> v_obs = this->obs();
  vector<GOBS>::iterator itOBS = v_obs.begin();
  for( ; itOBS != v_obs.end(); ++itOBS ) 
      cout << " " << t_gobs::gobs2str( *itOBS ) << ":" << this->getobs( *itOBS );
  cout << endl;

  cout << "GSATDATA - destruct KONEC: " << site() << " " << sat() << " "
       << epoch().str("  %Y-%m-%d %H:%M:%S[%T] ");
  cout.flush();   
#endif   
}


// -----------
void t_gsatdata::addcrd( const t_gtriple& crd )
{
  boost::mutex::scoped_lock lock(_mutex);
  _satcrd = crd;
}


// -----------
void t_gsatdata::addclk( double clk )
{
  boost::mutex::scoped_lock lock(_mutex);
  _clk = clk;
}


// -----------
void t_gsatdata::addele( double ele )
{
  boost::mutex::scoped_lock lock(_mutex);
  _ele = ele;
}


// -----------
void t_gsatdata::addazi( double azi )
{
  boost::mutex::scoped_lock lock(_mutex);
  _azi = azi;
}


// -----------
void t_gsatdata::addrho( double rho )
{
  boost::mutex::scoped_lock lock(_mutex);
  _rho = rho;
}


// -----------
t_gtriple t_gsatdata::satcrd()
{
  boost::mutex::scoped_lock lock(_mutex);
  t_gtriple crd(_satcrd[0], _satcrd[1], _satcrd[2] );
  return crd;
}


// -----------
double t_gsatdata::clk()
{
  boost::mutex::scoped_lock lock(_mutex);
  return _clk;
}


// -----------
double t_gsatdata::ele()
{
  boost::mutex::scoped_lock lock(_mutex);
  return _ele;
}


// -----------
double t_gsatdata::azi()
{
  boost::mutex::scoped_lock lock(_mutex);
  return _azi;
}


// -----------
double t_gsatdata::rho()
{
  boost::mutex::scoped_lock lock(_mutex);
  return _rho;
}


// valid
// ----------
bool t_gsatdata::valid()
{
  boost::mutex::scoped_lock lock(_mutex);
  return this->_valid();
}


// clean data
// ----------
void t_gsatdata::clear()
{ 
  boost::mutex::scoped_lock lock(_mutex);
  this->_clear();
}


// clean internal function
// ----------
void t_gsatdata::_clear()
{
  t_gobsgnss::_clear();
//  _satcrd[0] = 0.0;
//  _satcrd[1] = 0.0;
//  _satcrd[2] = 0.0;
  _ele = 0.0;
  _azi = 0.0;
  _rho = 0.0;
  _clk = 0.0;
}


// clean internal function
// ----------
bool t_gsatdata::_valid() const
{   
   if( _rho   == 0.0  ||   // single validity identification for gsatdata!
       t_gobsgnss::_valid() ) return false;

  return true; 
}
