
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <iostream>
#include <cmath>

#include "../../newmat/newmat.h"

#include "gmodels/gpar.h"
#include "gmodels/gmf.h"

using namespace std;

// constructor
// --------------------------------------------------------
t_gpar::t_gpar(string site, t_type t, int i, string p)
{
   beg = t_gtime::first_time(t_gtime::GPS);
   end = t_gtime::last_time(t_gtime::GPS);

   this->site = site;
   parType = t;
   index = i;
   prn = p;
   MWWL = 0.0;
   L4 = 0.0;
   MWslip = 0.0;
   value = 0.0;
}

t_gpar::t_gpar()
{
}

// t_gpar destructor
// ---------------------------------------------------
t_gpar::~t_gpar()
{
}

// setting mapping function for tropospherer paremetre
// ---------------------------------------------------
void t_gpar::setMF(string mf)
{   
  this->_mf = mf;
}

// get mapping function for tropospherer paremetre as sting
// ----------------------------------------------------
string t_gpar::getMF()
{  
  return this->_mf;
}

// Partial derivatives 
// ----------------------------------------------------
double t_gpar::partial(t_gsatdata& satData, t_gtime& epoch, double* ground)
{
  switch( parType ){
    case CRD_X : return (value - satData.satcrd().crd(0))/satData.rho();
    case CRD_Y : return (value - satData.satcrd().crd(1))/satData.rho();
    case CRD_Z : return (value - satData.satcrd().crd(2))/satData.rho();
    case CLK   : return 1.0; 
    case TRP   : 
           if (_mf.compare("cosz") == 0) {	     
	      return  1.0/sin(satData.ele());
	   
	   }else if (_mf.compare("gmf") == 0)  {	     
              
	      double gmfw, dgmfw, gmfh, dgmfh;	     
              gmf( epoch.mjd(), ground[0], ground[1], ground[2], PI/2.0-satData.ele(),
	           gmfh, gmfw, dgmfh, dgmfw );
	      return gmfw;
	   } else cerr << "Mapping functin is not set up correctly!!!" << endl;
    case AMB   : return 1.0;
  }
  return 0.0;
}


// Operators for t_gpar
// -------------------------------------------------
bool t_gpar::operator==(const t_gpar& par) const
{
   if ( parType == par.parType      &&
      //  prn.compare(par.prn)   == 0 &&
      //  site.compare(par.site) == 0 &&
	beg == par.beg &&
	end == par.end ){
      return true;
   } else {
      return false;
   }
   
}

// Doplnil gabo aby par mohla byt pouzita ako klic v mape
bool t_gpar::operator <(const t_gpar& par) const
{
    if (( parType <  par.parType )                                        ||
        ( parType == par.parType && prn <  par.prn )                      ||
        ( parType == par.parType && prn == par.prn && site <  par.site )  ||
        ( parType == par.parType && prn == par.prn && site == par.site && beg < par.beg ) ||
        ( parType == par.parType && prn == par.prn && site == par.site && beg == par.beg && end < par.end )
        ){
       return true;
    }
    return false;
}

// Setting begin and end time for validity object
// -------------------------------------------------
void t_gpar::setTime(const t_gtime& t1, const t_gtime& t2)
{
   this->beg = t1;
   this->end = t2;
}
