
#ifndef GPCV_H
#define GPCV_H 
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements antenna phase offsets and variations
  Version: $ Rev: $

  2011-12-05 /JD: created

-*/

#include <vector>
#include <boost/thread/mutex.hpp>

#include "../newmat/newmat.h"

#include "gdata/gdata.h"
#include "gutils/gconst.h"
#include "gutils/gfreq.h"
#include "gutils/gtime.h"
#include "gmodels/gephplan.h"

using namespace std;

// ----------
class t_gpcv : public t_gdata {

 public:
  t_gpcv();
//  t_gpcv(const t_gpcv& pcv);
  virtual ~t_gpcv();

  class triple { public: double neu[3]; };
   
//  typedef string                 t_gfreq::FRQ;
  typedef map<double, double>    t_map_Z;    // map of ZEN-dependence
  typedef map<double, t_map_Z>   t_map_A;    // map of AZI-dependence & ZEN-dependence
   
  typedef map<GFRQ, triple>    t_map_pco;    // North/East/Up phase centre offsets     (N-freqs)
  typedef map<GFRQ, t_map_Z>   t_map_zen;    // map of AZI-dependence                  (N-freqs)
  typedef map<GFRQ, t_map_A>   t_map_azi;    // map of AZI-dependence & ZEN-dependence (N-freqs)

  void   antenn(string s){ _antenn = s; }      // set/get antenn name
  string antenn()const{ return _antenn; }   
  void   radome(string s){ _radome = s; }      // set/get radome name
  string radome()const{ return _radome; }
  void   sernum(string s){ _sernum = s; }      // set/get serial number
  string sernum()const{ return _sernum; }
  void   method(string s){ _method = s; }      // set/get method of callibration
  string method()const{ return _method; }
  void   source(string s){ _source = s; }      // set/get source of callibration
  string source()const{ return _source; }
  void   snxcod(string s){ _snxcod = s; }      // set/get sinex code
  string snxcod()const{ return _snxcod; }

  void    beg(t_gtime t){ _beg = t; }          // set/get valid from
  t_gtime beg()const{ return _beg;  }
  void    end(t_gtime t){ _end = t; }          // set/get valid until
  t_gtime end()const{ return _end;  }

  void   dazi(double d){ _dazi = d; }          // set/get azimuth sampling
  double dazi()const{ return _dazi; }
  void   dzen(double d){ _dzen = d; }          // set/get zenith sampling
  double dzen()const{ return _dzen; }
  void   zen1(double d){ _zen1 = d; }          // set/get zenith1
  double zen1()const{ return _zen1; }
  void   zen2(double d){ _zen2 = d; }          // set/get zenith2
  double zen2()const{ return _zen2; }

  // PV - docasna implementace !!!!!
  int       pco( t_gtime& epo, ColumnVector& crdSat, GFRQ f);
  double    pco( double zen, double azi, GFRQ f ); // get correction: zen/azi [rad]
  double    pcv( double zen, double azi, GFRQ f ); // get correction: zen/azi [rad]
  double    pcv( double zen,             GFRQ f ); // get correction: zen [rad]
  // -----
  
  void      pco(GFRQ f,   const triple& t ){ _mappco[f] = t; }  // set/get PCO
  triple    pco(GFRQ f){              return _mappco[f]; }      // CANNOT BE CONST ?
  t_map_pco pco() const{              return _mappco; }
   
  void      pcvzen(GFRQ f,const t_map_Z& t){ _mapzen[f] = t; }  // set/get PCO
  t_map_Z   pcvzen(GFRQ f){           return _mapzen[f]; }      // CANNOT BE CONST ?
  t_map_zen pcvzen() const{           return _mapzen; }
   
  void      pcvazi(GFRQ f,const t_map_A& t){ _mapazi[f] = t; }  // set/get PCO
  t_map_A   pcvazi(GFRQ f){           return _mapazi[f]; }      // CANNOT BE CONST ?
  t_map_azi pcvazi() const{           return _mapazi; }

 private:

  t_gephplan     _ephplan;
  string         _antenn;     // antenn name
  string         _radome;     // radome name
  string         _sernum;     // serial number
  string         _method;     // method of calibration
  string         _source;     // source of calibration
  string         _snxcod;     // sinex code
  t_gtime        _beg;        // valid from
  t_gtime        _end;        // valid until
  double         _dazi;       // azimuth sampling
  double         _dzen;       // zenith sampling
  double         _zen1;       // zenith1
  double         _zen2;       // zenith2

  t_map_pco      _mappco;     // map of PCOs (all frequencies)
  t_map_zen      _mapzen;     // map of NOAZI values (all frequencies)
  t_map_azi      _mapazi;     // map of AZI-dep values (all frequencies)

};


#endif
