
#ifndef GPPPMODEL_H
#define GPPPMODEL_H

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: various PPP models
  Version: $ Rev: $

  2012-09-20 /PV: created

-*/

#include <string>
#include <map>
#include <cmath>

#include "../newmat/newmat.h"
#include "../newmat/newmatio.h"

#include "gset/gsetproc.h"
#include "gset/gsetrec.h"
#include "gio/glog.h"
#include "gutils/gsysconv.h"
#include "gutils/gtypeconv.h"
#include "gutils/gtime.h"
#include "gdata/gsatdata.h"
#include "gall/gallpar.h"
#include "gmodels/gmf.h"
#include "gmodels/gtropo.h"
#include "gmodels/gephplan.h"

using namespace std;

class t_gpppmodel
{
 public:
   t_gpppmodel(string site,t_glog* glog, t_gsetbase* settings);
   t_gpppmodel() {};
  ~t_gpppmodel();
   
   void   glog(t_glog* l){ _log = l; }
   double windUp(t_gtime& epoch, const string, const ColumnVector&, const ColumnVector&);
   int outlierDetect( vector<t_gsatdata>& data, SymmetricMatrix& Qx, const SymmetricMatrix&, const ColumnVector&);   
   double cmpObs(t_gtime& epoch, t_gallpar& param, t_gsatdata&, bool);
   double tropoDelay(t_gtime& epoch, t_gallpar& param, t_gtriple ell, double elevation);
   t_gtropo* tropoModel;
   void setSite(string site);
   
  private:   
   boost::mutex*       _mutex;
   t_gsetbase*         _settings;
   map<string, double> _windUpTime;
   map<string, double> _windUpSum;
   string              _site;
   t_glog*             _log;
   t_gephplan          _ephplan;
   
};


#endif //  GPPPMODEL_H
