
#ifndef KALMAN_H
#define KALMAN_H 
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements Kalman filter class
  Version: $ Rev: $

  2011-01-10 /JD: created

-*/

#include "../newmat/newmat.h"

#include "gmodels/gbancroft.h"
#include "gmodels/gpar.h"
#include "gdata/gsatdata.h"

using namespace std;

// Base class for all filtering technique
// -------------------------------------------------------

class t_gfilter 
{
 public:
   t_gfilter();
   virtual ~t_gfilter();
   
   virtual void update(const Matrix& A, const DiagonalMatrix& P, const ColumnVector& l, ColumnVector& dx, SymmetricMatrix& Q) {};
   virtual void update_AmbEl(const Matrix& A, const DiagonalMatrix& P, const ColumnVector& l, ColumnVector& dx, const SymmetricMatrix& Q, SymmetricMatrix& Q_el) {};
//   virtual void smooth(const gpar& Xpre, const gpar& Xupd, const SymmetricMatrix& Qpre, const SymmetricMatrix& Qupd, gpar& Xsm, SymmetricMatrix& Qsm) {};
 protected:
};

// --------------------------------------------------------
 

// Classical formule for Kalman filter
// -------------------------------------------------------
class t_kalman : public t_gfilter
{
   
 public:

  t_kalman();
 ~t_kalman();
   
 void update(const Matrix& A, const DiagonalMatrix& P, const ColumnVector& l, ColumnVector& dx, SymmetricMatrix& Q);
 void update_AmbEl(const Matrix& A, const DiagonalMatrix& P, const ColumnVector& l, ColumnVector& dx, const SymmetricMatrix& Q, SymmetricMatrix& Q_el);
   
 private:
   Matrix ortg(const Matrix& A, const DiagonalMatrix& P);
};
//-------------------------------------------------------

// Square root covariance filter
// ------------------------------------------------------
class t_SRF : public t_gfilter
{
 public:
   t_SRF();
   ~t_SRF();
   
  void update(const Matrix& A, const DiagonalMatrix& P, const ColumnVector& l, ColumnVector& dx, SymmetricMatrix& Q);
};
// ------------------------------------------------------
 
// Square root information filter
// ------------------------------------------------------
class t_SRIF : public t_gfilter
{
 public:
   t_SRIF();
   ~t_SRIF();
   
   void update(const Matrix& A, const DiagonalMatrix& P, const ColumnVector& l, ColumnVector& dx, SymmetricMatrix& Q);
};
// -------------------------------------------------------

#endif

