
#ifndef GPPPFILTER_H
#define GPPPFILTER_H

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements ppp client
  Version: $ Rev: $

  2011-06-06 /PV: created
  2011-06-06 /JD: revised for multi-threading

-*/

#include <fstream>
#include <boost/thread/mutex.hpp>

#include "../newmat/newmat.h"
#include "../newmat/newmatio.h"

#include "gio/glog.h"
#include "gio/giof.h"
#include "gproc/gfilter.h"
#include "gdata/gsatdata.h"
#include "gall/gallobs.h"
#include "gall/gallotl.h"
#include "gall/gallpcv.h"
#include "gmodels/gstochasticmodel.h"
#include "gmodels/gpar.h"
#include "gmodels/gtropo.h"
#include "gmodels/gpppmodel.h"
#include "gmodels/gtide.h"
#include "gmodels/gtide96.h"
#include "gmodels/gephplan.h"

#include "gset/gsetflt.h"
#include "gset/gsetout.h"
#include "gset/gsetproc.h"

class t_gpppfilter
{
 public:
   t_gpppfilter();      
   t_gpppfilter(string mark, t_gsetbase* set);
  ~t_gpppfilter();
    
   void   processBatch(const t_gtime& beg, const t_gtime& end, int sampling = 1);
   void   setDAT(t_gallobs* gobs, t_gallnav* gnav);
   void   setPCV(t_gallpcv* gpcv);
   void   setOTL(t_gallotl* gotl);
   
   bool   isRunning();
   
 protected:
   void   _filterData(vector<t_gsatdata>& data);
   int    _processEpoch(const t_gtime& runEpoch);
   void   _predict();
   int    _satPos(t_gtime&, t_gsatdata&);
   int    _prepareData();
   void   _syncAmb();
   void   _setLog();
   void   _setOut();

   string _site;

   t_gsetbase*    _set;
   t_glog*        _log;
   t_giof*        _flt;
   t_giof*        _smt;

   bool    _phase;
   bool    _smooth;
   bool    _running;
   bool    _initialized;      
   int     _cntrep;
   double  _sigCode;
   double  _sigPhase;
   double  _sigAmbig;

   vector<t_gsatdata> _data;
   
// Parameters and covariance matrix   
   t_gallpar           _param; 
   SymmetricMatrix     _Qx;
   
// Models   
   t_gpppmodel*        _gpppModel;
   t_randomwalk*       _trpStoModel;
   t_whitenoise*       _clkStoModel;
   t_whitenoise*       _crdStoModel;   
   
// Epoch time
   t_gtime           _epoch; 
   
// Time for constraining ZTD
   t_gtime           _cEpo_beg; 
   t_gtime           _cEpo_end;    
   
// Estimation objects   
   t_gfilter*        _filter;
   
// Observation data   
   t_gallobs*        _epodata;
   
// Ocean loading date
   t_gallotl*        _gotl;
   
// Objects for ephemerides
   t_gallnav*        _gnav;
   
// Tides
   t_gtide*          _tides;
   
// Phase center ofsets and variation   
   t_gallpcv*        _gpcv;

   ColumnVector      _vBanc;

   boost::mutex      _mutex;
   
};


#endif // GPPP_H
