
#ifndef PREPROCESSING_H
#define PREPROCESSING_H

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: preprocessing, header
  Version: $ Rev: $

  2012-09-03 /PV: created

-*/

#include <sstream>

#include "../newmat/newmat.h"

#include "gio/glog.h"
#include "gall/gallobs.h"
#include "gall/gallnav.h"
#include "gset/gsetproc.h"
#include "gutils/gfreq.h"


class t_gpreproc : public t_gmonit
{
 public:
   t_gpreproc(t_gallobs* obs, t_gsetbase* settings);
   ~t_gpreproc();

   void glog(t_glog* l){ _log = l; }                 // set/get glog pointer
   t_glog* glog()const{ return _log; }

   int preprocess(string site, const t_gtime& beg, const t_gtime& end, int sampl = 1);
   void jumpLog(t_glog* glog);
   void slipLog(t_glog* glog);
   void setNav(t_gallnav* nav);
   void logging(t_glog* glog);
   void filterData(map<string, t_gobsgnss>* data);
   void setSite(string site);

 private:
   t_gallobs*         _obs;
   t_gallnav*         _nav;   
   string             _site;
   t_glog*            _log;
   t_gsetbase*        _settings;
   double             _sigCode, _sigPhase;
   double             _sumS;   
   t_gtriple          _pos;
   
   map<string, int>   _msoffset;
   map<t_gtime, int>  _mbreaks;                               // for logging
   map<t_gtime, map<string,pair<string,string> > > _mslips;   // for logging

   double _slipThreshold(string LC, t_gsys* gsys, double sampl = 30.0);
   int _slips(t_gobsgnss* gobs1, t_gobsgnss* gobs2);
   int _jumps(t_gobsgnss* gobs1, t_gobsgnss* gobs2);
   double _elevation(const string& site, t_gtime epoch, string prn);
   void _repair(map<string, t_gobsgnss>* epoData, double dL);
};


#endif
