
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <iomanip>
#include <sstream>
#include <boost/thread/mutex.hpp>

#include "gset/gcfg_gebpp.h"

using namespace std;
using namespace pugi;

// Constructor
// ----------
t_gcfg_gebpp::t_gcfg_gebpp() 
 : t_gsetbase(),
   t_gsetgen(),
   t_gsetinp(),
   t_gsetout(),
   t_gsetflt()
{
  _IFMT_supported.insert(RINEXO_INP);
  _IFMT_supported.insert(RINEXC_INP);
  _IFMT_supported.insert(ATX_INP);
  _IFMT_supported.insert(SP3_INP);

  _OFMT_supported.insert(LOG_OUT);
  _OFMT_supported.insert(PPP_OUT);
  _OFMT_supported.insert(FLT_OUT);
   
  _useGLONASS     = false;
  _sig_init_ztd   = 0.1;
  _tropo_model    = "saastamoinen";
  _tropo_mf       = "gmf";
}


// Destructor
// ----------
t_gcfg_gebpp::~t_gcfg_gebpp()
{}


// settings check
// ----------
void t_gcfg_gebpp::check()
{
  t_gsetgen::check();
  t_gsetinp::check();
  t_gsetout::check();
  t_gsetrec::check();
  t_gsetproc::check();
  t_gsetflt::check();
   
  xml_node node;

   node = _doc.child(XMLKEY_ROOT).child(XMLKEY_FLT);
  _default_attr(node, "rndwk_ztd", 1, true);

   node = _doc.child(XMLKEY_ROOT).child(XMLKEY_PROC);
  _default_attr(node, "tropo_model",  _tropo_model,  true);
  _default_attr(node, "tropo_mf",     _tropo_mf,     true);
  _default_attr(node, "sig_init_ztd", _sig_init_ztd, true);

#ifdef DEBUG   
  ostringstream os;
  write_ostream(os);
  cerr << os.str() << endl;
  exit(1);
#endif
}

// settings help
// ----------
void t_gcfg_gebpp::help()
{
  t_gsetbase::help_header();
  t_gsetgen::help();
  t_gsetinp::help();
  t_gsetout::help();
  t_gsetrec::help();
  t_gsetproc::help();
  t_gsetflt::help();
  t_gsetbase::help_footer();
}
