
#ifndef GSETBASE_H
#define GSETBASE_H

#define XMLKEY_ROOT "config"

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements base setting class
  Version: $ Rev: $

  2012-10-23 /JD: created

-*/

#include <set>
#include <string>
#include <iostream>
#include <boost/thread/mutex.hpp>

#include "gio/glog.h"
#include "gutils/gtypeconv.h"
#include "pugixml/src/pugixml.hpp"

using namespace std;
using namespace pugi;

class t_gsetbase : public pugi::xml_node
{
 public:
   t_gsetbase();
  ~t_gsetbase();
  
   virtual int glog(t_glog* glog);               // read configuration glog and set it
   
   int read(const string& file);                 // read configuration from file
   int read_istream(istream& is);                // read configuration from istream
   int write(const string& file);                // write configuration in file
   int write_ostream(ostream& os);               // write configuration in ostream

   virtual void check() = 0;                     // settings check
   virtual void help() = 0;                      // settings help

   string         strval(const string& elem, const string& subelem); // public interface
   set<string>    setval(const string& elem, const string& subelem); // public interface
   vector<string> vecval(const string& elem, const string& subelem); // public interface

 protected:
   string         _strval(const string& elem, const string& subelem);
   set<string>    _setval(const string& elem, const string& subelem);
   vector<string> _vecval(const string& elem, const string& subelem);

//   xml_node _default_node(xml_node& node, const char* n);
   xml_node _default_node(xml_node& node, const char* n, const char* val="", bool reset=false);

   void _default_attr(xml_node& node, const char* n, const string& v,     bool reset=false);
   void _default_attr(xml_node& node, const char* n, const bool& value,   bool reset=false);
   void _default_attr(xml_node& node, const char* n, const int& value,    bool reset=false);
   void _default_attr(xml_node& node, const char* n, const double& value, bool reset=false);

   virtual void help_header();                   // XML settings header
   virtual void help_footer();                   // XML settings footer

   xml_document     _doc;                        // root document
   xml_parse_result _irc;                        // result status
   
   t_glog*        _log;                          // pointer to log file
   string         _name;                         // configuration name
   string         _delimiter;                    // delimiter for writting nodes/elements
   
   boost::mutex   _mutex;                        // muttable access
   
 private:

};

#endif
