
#ifndef GSETOUT_H
#define GSETOUT_H

#define XMLKEY_OUT "outputs"

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements output setting class
  Version: $ Rev: $

  2012-10-23 /JD: created

-*/

#include <string>
#include <iostream>
#include <boost/thread/mutex.hpp>

#include "gio/glog.h"
#include "gutils/gtypeconv.h"
#include "gsetbase.h"

using namespace std;
using namespace pugi;

// the order is important here!
enum OFMT {
     XXX_OUT,
     LOG_OUT,
     XTR_OUT,
     PPP_OUT,
     FLT_OUT,
     SMT_OUT
};

class t_gsetout : public virtual t_gsetbase
{
 public:
   t_gsetout();
  ~t_gsetout();

   static OFMT   str2ofmt( const string& s );
   static string ofmt2str( const OFMT&   f );

   int glog(t_glog* glog);                     // read configuration log & set it
     
   void check();                               // settings check
   void help();                                // settings help

   // attributes
   int verb();                                 // get verbosity attribute
   bool append();                              // get append request

   // elements
   string outputs(const string& fmt);          // get string outputs
   set<string> oformats();                     // get formats

 protected:
   string _outputs(const string& fmt);
   set<string> _oformats();

   set<OFMT> _OFMT_supported;                  // vector of supported OFMTs (app-specific)
   bool _append;                               // append mode
   int  _verb;                                 // output verbosity   
   
 private:

};

#endif
