
#ifndef GSETPROC_H
#define GSETPROC_H

#define XMLKEY_PROC "process"

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements processing setting
  Version: $ Rev: $

  2012-10-23 /JD: created

-*/

#include <string>
#include <iostream>
#include <boost/thread/mutex.hpp>

#include "gio/glog.h"
#include "gutils/gtypeconv.h"
#include "gset/gsetbase.h"

using namespace std;
using namespace pugi;

class t_gsetproc : public virtual t_gsetbase
{
 public:
   t_gsetproc();
  ~t_gsetproc();
  
   void check();                                 // settings check
   void help();                                  // settings help

   bool   tropo();
   bool   phase();
   double sigma_C();
   double sigma_L();
   double sig_init_ztd();
   double sig_init_crd();
   double sig_init_amb();   
   double minimum_elev();
   string tropo_mf();
   string tropo_model();
   bool useGLONASS();
   
 protected:
   
   bool   _phase;                                // use phase data [true=1, false=0]
   bool   _tropo;                                // estimate troposphere [true=1, false=0]
   string _tropo_model;                          // tropo model [SAAS, GPT, VMF1, ... ]
   string _tropo_mf;                             // tropo mapping function [COSZ, NIELL, GMF, VMF1, ... ]
   double _sigma_C;                              // C/code measurements precision [m]
   double _sigma_L;                              // L/phase measurements precision [m]
   double _sig_init_ztd;                         // accuracy of initial zenith path delays [m]
   double _sig_init_crd;                         // accuracy of initial coordinates [m]
   double _sig_init_amb;                         // accuracy of initial ambiguity [m]   
   double _minimum_elev;                         // elevation angle cut-off [degree]
   bool   _useGLONASS;                           // use GLONASS
   
 private:
};

#endif
