
#ifndef GSETREC_H
#define GSETREC_H

#define XMLKEY_REC "rec"

/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: implements receiver object settings
  Version: $ Rev: $

  2012-10-23 /JD: created

-*/

#include <string>
#include <iostream>
#include <boost/thread/mutex.hpp>

#include "gio/glog.h"
#include "gset/gsetbase.h"
#include "gutils/gtypeconv.h"
#include "gutils/gtriple.h"

using namespace std;

class t_gsetrec : public virtual t_gsetbase
{
 public:
   t_gsetrec();
  ~t_gsetrec();

   void check();                                  // settings check
   void help();                                   // settings help

   string id();
// Apriori information
   double X();
   double Y();
   double Z();
   double dX();
   double dY();
   double dZ();
   double ZTD();
   double ZTD_constr();
   string CRD_constr();
   int getCRD(t_gtriple& crd); 
   bool valid_crd();
   bool valid_ztd();

 protected:

   string _id;                                    // receiver id+domex
   string _name;                                  // receiver full name
   double _X;                                     // receiver X-coordinate [m] 
   double _Y;                                     // receiver Y-coordinate [m] 
   double _Z;                                     // receiver Z-coordinate [m]
   double _dX;                                    // receiver X-eccentricity [m]
   double _dY;                                    // receiver Y-eccentricity [m] 
   double _dZ;                                    // receiver Z-eccentricity [m]
   double _ZTD;
   double _ZTD_constr;
   string _CRD_constr;
   
 private:
};

#endif
