
#ifndef GFREQ_H
#define GFREQ_H
 
/* ----------------------------------------------------------------------
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  Purpose: definition of GNSS observation frequencies
  Version: $ Rev: $

  2012-09-26 /JD: created

-*/

#include <string>
#include <boost/thread/mutex.hpp>

#include "gutils/gconst.h"

using namespace std;

// GNSS base frequencies [Hz]
// --------------------------
#define  GPS_FRQ      10230000.0       // GPS     base frequency
#define  GLO_FRQ     178000000.0       // GLONASS base frequency
#define  GAL_FRQ      10230000.0       // GALILEO base frequency
#define  BDS_FRQ      10230000.0       // BEIDOU  base frequency
#define  QZSS_FRQ     10230000.0       // QZSS    base frequency
#define  SBAS_FRQ     10230000.0       // SBAS    base frequency

// GNSS frequency multiplier factors (dimensionless)
// -------------------------------------------------
#define  G01_MLT          154.0
#define  G02_MLT          120.0
#define  G05_MLT          115.0
#define  R01_MLT            9.0
#define  R02_MLT            7.0
#define  R01_STEP      562500.0
#define  R02_STEP      437500.0
#define  E01_MLT          154.0
#define  E05_MLT          115.0
#define  E06_MLT          125.0
#define  E07_MLT          118.0
#define  E08_MLT          116.5
#define  C01_MLT          155.4   // or 154 ????
#define  C02_MLT          152.6  
#define  C06_MLT          124.0
#define  C07_MLT          118.0
#define  J01_MLT            0.0
#define  J02_MLT            0.0
#define  J05_MLT            0.0
#define  J06_MLT            0.0
#define  S01_MLT          154.0
#define  S05_MLT          115.0

// GNSS derived frequencies [Hz]
// -----------------------------
#define  G01_F      G01_MLT * GPS_FRQ                //  L1    - GPS NAVSTAR
#define  G02_F      G02_MLT * GPS_FRQ                //  L2    - GPS NAVSTAR
#define  G05_F      G05_MLT * GPS_FRQ                //  L5    - GPS NAVSTAR

#define  R01_F(a)   R01_MLT * GLO_FRQ + R01_STEP*(a) //  G1    - GLONASS base frequency
#define  R02_F(a)   R02_MLT * GLO_FRQ + R02_STEP*(a) //  G2    - GLONASS base frequency

#define  E01_F      E01_MLT * GAL_FRQ                //  E1    - Galileo
#define  E05_F      E05_MLT * GAL_FRQ                //  E5a   - Galileo
#define  E06_F      E06_MLT * GAL_FRQ                //  E6    - Galileo
#define  E07_F      E07_MLT * GAL_FRQ                //  E5b   - Galileo
#define  E08_F      E08_MLT * GAL_FRQ                //  E5a+b - Galileo

#define  C01_F      C01_MLT * BDS_FRQ                //  E1    - BeiDou
#define  C02_F      C02_MLT * BDS_FRQ                //  E2    - BeiDou
#define  C06_F      C06_MLT * BDS_FRQ                //  E6    - BeiDou
#define  C07_F      C07_MLT * BDS_FRQ                //  E5b   - BeiDou

#define  J01_F      J01_MLT * QZSS_FRQ               //  L1    - QZSS
#define  J02_F      J02_MLT * QZSS_FRQ               //  L2    - QZSS
#define  J05_F      J05_MLT * QZSS_FRQ               //  L5    - QZSS
#define  J06_F      J06_MLT * QZSS_FRQ               //  LEX   - QZSS

#define  S01_F      S01_MLT * SBAS_FRQ               //  L1    - SBAS
#define  S05_F      S05_MLT * SBAS_FRQ               //  L5    - SBAS

/*
#define  G01_F     1575420000.0       //  L1    - GPS NAVSTAR
#define  G02_F     1227600000.0       //  L2    - GPS NAVSTAR
#define  G05_F     1176450000.0       //  L5    - GPS NAVSTAR
#define  R01       1602000000.0       //  G1    - GLONASS base frequency
#define  R02       1246000000.0       //  G2    - GLONASS base frequency
#define dR01           562500.0       // dG1    - GLONASS frequency step for channels
#define dR02           437500.0       // dG2    - GLONASS frequency step for channels
#define  R01_F(a)  R01+(a)*dR01       //  G1    - GLONASS channel-specific frequency
#define  R02_F(a)  R02+(a)*dR02       //  G2    - GLONASS channel-specific frequency
#define  E01_F     1575420000.0       //  E1    - Galileo       
#define  E05_F     1176450000.0       //  E5a   - Galileo       
#define  E06_F     1278750000.0       //  E6    - Galileo       
#define  E07_F     1207140000.0       //  E5b   - Galileo       
#define  E08_F     1191795000.0       //  E5a+b - Galileo       
#define  C01_F     1589740000.0       //  E1    - BeiDou       
#define  C02_F     1561098000.0       //  E2    - BeiDou       
#define  C07_F     1207140000.0       //  E5b   - BeiDou       
#define  C06_F     1268520000.0       //  E6    - BeiDou
#define  J01_F              0.0       //  L1    - QZSS          
#define  J02_F              0.0       //  L2    - QZSS          
#define  J05_F              0.0       //  L5    - QZSS
#define  J06_F              0.0       //  LEX   - QZSS
#define  S01_F     1575420000.0       //  L1    - SBAS
#define  S05_F     1176450000.0       //  L5    - SBAS
*/

// GNSS wave lengths [m]
// ---------------------
/*
#defined G01_WL     CLIGHT/G01_F
#defined G02_WL     CLIGHT/G02_F
#defined G05_WL     CLIGHT/G05_F
#defined R01_WL(a)  CLIGHT/R01_F(a)
#defined R02_WL(a)  CLIGHT/R02_F(a)
#defined E01_WL     CLIGHT/E01_F
#defined E02_WL     CLIGHT/E02_F
#defined E05_WL     CLIGHT/E05_F
#defined E06_WL     CLIGHT/E06_F
#defined E07_WL     CLIGHT/E07_F
#defined E08_WL     CLIGHT/E08_F
#defined C01_WL     CLIGHT/C01_F
#defined C02_WL     CLIGHT/C02_F
#defined C06_WL     CLIGHT/C06_F
#defined C07_WL     CLIGHT/C07_F
#defined J01_WL     CLIGHT/J01_F
#defined J02_WL     CLIGHT/J02_F
#defined J05_WL     CLIGHT/J05_F
#defined J06_WL     CLIGHT/J06_F
#defined S01_WL     CLIGHT/S01_F
#defined S05_WL     CLIGHT/S05_F
*/

// OBSOLETE INFORMATIONS
#define GPSLAMB1    CLIGHT/(G01_F)     // GPS L1 wave length [m]
#define GPSLAMB2    CLIGHT/(G02_F)     // GPS L2 wave length [m]
#define GPSLAMB3   ( ((G01_F*G01_F) / (G01_F*G01_F - G02_F*G02_F)) * GPSLAMB1 - ((G02_F*G02_F) / (G01_F*G01_F - G02_F*G02_F)) * GPSLAMB2 )  // GPS ionospher-free wave lenght [m]
#define LAMBWL      CLIGHT/(G01_F-G02_F)   // GPS wide-lane wave length [m]

//#define GPSFREG1    1575420000.0        // GPS L1 frequency [Hz]
//#define GPSFREG2    1227600000.0        // GPS L2 grequency [Hz]
//#define GPSLAMB1_1    CLIGHT/GPSFREG1     // GPS L1 wave length [m]
//#define GPSLAMB2_1    CLIGHT/GPSFREG2     // GPS L2 wave length [m]
//#define GPSLAMB3_1 (((GPSFREG1*GPSFREG1)/(GPSFREG1*GPSFREG1-GPSFREG2*GPSFREG2)) * GPSLAMB1) - (((GPSFREG2*GPSFREG2)/(GPSFREG1*GPSFREG1-GPSFREG2*GPSFREG2)) * GPSLAMB2)
//#define LAMBWL_1      (CLIGHT/(GPSFREG1-GPSFREG2))   // wide-lane wave length [m]




// GNSS systems and augmentations
// ------------------------------
enum GSYS { // GXX = -1,
            GPS, GAL, GLO, BDS, QZSS, SBAS, LAST_GSYS };

// GNSS frequencies
// -----------------
enum GFRQ { // FXX = -1,
            G01, G02, G05,            // GPS
            R01, R02,                 // GLONASS
            E01, E05, E07, E08, E06,  // Galileo
            C01, C02, C07, C06,       // BeiDou
            J01, J02, J05, J06,       // QZSS
            S01, S05,                 // SBAS
            LAST_GFRQ
          };

// class 
// ---------- 
class t_gsys {
   
 public:
   t_gsys(GSYS sys);
   t_gsys(string sys);
   t_gsys(char c);
  ~t_gsys(){};

   // general (static) functions
   static string gsys2str(GSYS sys);             // convert GSYS enum to GSYS string
   static char gsys2char(GSYS sys);              // convert GSYS enum to GSYS char
   static GSYS str2gsys(string s);               // convert GSYS string to GSYS enum
   static char str2char(string s);               // convert GSYS string to GSYS char
   static string char2str(char c);               // convert GSYS char to GSYS string
   static GSYS char2gsys(char c);                // convert GSYS char to GSYS enum

   // convert satellite name
   static string eval_sat(string sat, GSYS sys = GPS);  // convert full sat string

// static double frequency(  int i, string sat); // get sat-specific frequency  for band i
// static double wavelength( int i, string sat); // get sat-specific wavelength for band i
   
   double factor( int i );                       // get system-specific factor     for band i
   double frequency( int i );                    // get system-specific frequency  for band i 
   double wavelength( int i );                   // get system-specific wavelength for band i

   void   from_string( string sys );             // set GSYS from string
   void   from_gsys( GSYS     sys );             // set GSYS from enum
 
   GSYS   gsys()const{return _gsys;}             // get GSYS enum
   string  str()const{return gsys2str(_gsys);}   // get GSYS string
   
   bool operator==(const string& sys) const;     // overloaded equivalence operator
   bool operator==(const GSYS&   sys) const;     // overloaded equivalence operator
   
 protected:
// GSYS   _to_gsys( string sys );                // internal conversion function
// string _to_string( GSYS sys );                // internal conversion function
   
 private:
   GSYS            _gsys;
   boost::mutex   _mutex;
};


// class
// ----------
class t_gfreq {

 public:
   t_gfreq(){};
  ~t_gfreq(){};

   static GFRQ   str2gfreq( string frq );      // get GFRQ enum from string
   static string gfreq2str( GFRQ frq );        // get string from GFRQ enum
   
//   static lambda_L1( GSYS )

 private:
   GFRQ    _gfreq;
//   static boost::mutex   _mutex;

};


#endif // GFREQ_H