
/* ----------------------------------------------------------------------
 * G-Nut - GNSS software development library
 * 
  (c) 2011 Geodetic Observatory Pecny, http://www.pecny.cz (gnss@pecny.cz)
      Research Institute of Geodesy, Topography and Cartography
      Ondrejov 244, 251 65, Czech Republic

  This file is part of the G-Nut C++ library.
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 3 of
  the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, see <http://www.gnu.org/licenses>.

-*/

#include <iostream>
#include <iomanip>

#include "gutils/gfreq.h"
#include "gutils/gobs.h"

// using namespace std;

// convert obs code string to GOBS
// -----------
GOBS t_gobs::str2gobs( const string& o )
{
// STATIC FUNCTION !!!
//  boost::mutex::scoped_lock lock(_mutex);

//  if( obs.compare("XXX") == 0 ){ return X; }

  int len = o.length();
  if( len == 3 ){

    if( o[0] == 'C' ){ // code       
      if( o[1] == '1' ){ // band
        if( o[2] == 'A' ) return C1A;
        if( o[2] == 'B' ) return C1B;
        if( o[2] == 'C' ) return C1C;
        if( o[2] == 'L' ) return C1L;
        if( o[2] == 'M' ) return C1M;
        if( o[2] == 'P' ) return C1P;
        if( o[2] == 'S' ) return C1S;
        if( o[2] == 'W' ) return C1W;
        if( o[2] == 'X' ) return C1X;
        if( o[2] == 'Y' ) return C1Y;
        if( o[2] == 'Z' ) return C1Z;
      }
      if( o[1] == '2' ){ // band
        if( o[2] == 'C' ) return C2C;
        if( o[2] == 'D' ) return C2D;
        if( o[2] == 'I' ) return C2I;
        if( o[2] == 'L' ) return C2L;
        if( o[2] == 'M' ) return C2M;
        if( o[2] == 'P' ) return C2P;
        if( o[2] == 'Q' ) return C2Q;
        if( o[2] == 'S' ) return C2S;
        if( o[2] == 'W' ) return C2W;
        if( o[2] == 'X' ) return C2X;
        if( o[2] == 'Y' ) return C2Y;
      }
      if( o[1] == '5' ){ // band
        if( o[2] == 'I' ) return C5I;
        if( o[2] == 'Q' ) return C5Q;
        if( o[2] == 'X' ) return C5X;
      }
      if( o[1] == '6' ){ // band
        if( o[2] == 'A' ) return C6A;
        if( o[2] == 'B' ) return C6B;
        if( o[2] == 'C' ) return C6C;
        if( o[2] == 'I' ) return C6I;
        if( o[2] == 'Q' ) return C6Q;
        if( o[2] == 'X' ) return C6X;
        if( o[2] == 'Z' ) return C6Z;
      }
      if( o[1] == '7' ){ // band
        if( o[2] == 'I' ) return C7I;
        if( o[2] == 'Q' ) return C7Q;
        if( o[2] == 'X' ) return C7X;
      }
      if( o[1] == '8' ){ // band
        if( o[2] == 'I' ) return C8I;
        if( o[2] == 'Q' ) return C8Q;
        if( o[2] == 'X' ) return C8X;
      }
    }
     
    if( o[0] == 'L' ){ // phase
      if( o[1] == '1' ){ // band
        if( o[2] == 'A' ) return L1A;
        if( o[2] == 'B' ) return L1B;
        if( o[2] == 'C' ) return L1C;
        if( o[2] == 'L' ) return L1L;
        if( o[2] == 'M' ) return L1M;
        if( o[2] == 'N' ) return L1N;
        if( o[2] == 'P' ) return L1P;
        if( o[2] == 'S' ) return L1S;
        if( o[2] == 'W' ) return L1W;
        if( o[2] == 'X' ) return L1X;
        if( o[2] == 'Y' ) return L1Y;
        if( o[2] == 'Z' ) return L1Z;
      }
      if( o[1] == '2' ){ // band
        if( o[2] == 'C' ) return L2C;
        if( o[2] == 'D' ) return L2D;
        if( o[2] == 'I' ) return L2I;
        if( o[2] == 'L' ) return L2L;
        if( o[2] == 'M' ) return L2M;
        if( o[2] == 'N' ) return L2N;
        if( o[2] == 'P' ) return L2P;
        if( o[2] == 'Q' ) return L2Q;
        if( o[2] == 'S' ) return L2S;
        if( o[2] == 'W' ) return L2W;
        if( o[2] == 'X' ) return L2X;
        if( o[2] == 'Y' ) return L2Y;
      }
      if( o[1] == '5' ){ // band
        if( o[2] == 'I' ) return L5I;
        if( o[2] == 'Q' ) return L5Q;
        if( o[2] == 'X' ) return L5X;
      }
      if( o[1] == '6' ){ // band
        if( o[2] == 'A' ) return L6A;
        if( o[2] == 'B' ) return L6B;
        if( o[2] == 'C' ) return L6C;
        if( o[2] == 'I' ) return L6I;
        if( o[2] == 'Q' ) return L6Q;
        if( o[2] == 'X' ) return L6X;
        if( o[2] == 'Z' ) return L6Z;
      }
      if( o[1] == '7' ){ // band
        if( o[2] == 'I' ) return L7I;
        if( o[2] == 'Q' ) return L7Q;
        if( o[2] == 'X' ) return L7X;
      }
      if( o[1] == '8' ){ // band
        if( o[2] == 'I' ) return L8I;
        if( o[2] == 'Q' ) return L8Q;
        if( o[2] == 'X' ) return L8X;
      }
    }
   
    if( o[0] == 'D' ){ // doppler       
      if( o[1] == '1' ){ // band
        if( o[2] == 'A' ) return D1A;
        if( o[2] == 'B' ) return D1B;
        if( o[2] == 'C' ) return D1C;
        if( o[2] == 'L' ) return D1L;
        if( o[2] == 'M' ) return D1M;
        if( o[2] == 'N' ) return D1N;
        if( o[2] == 'P' ) return D1P;
        if( o[2] == 'S' ) return D1S;
        if( o[2] == 'W' ) return D1W;
        if( o[2] == 'X' ) return D1X;
        if( o[2] == 'Y' ) return D1Y;
        if( o[2] == 'Z' ) return D1Z;
      }
      if( o[1] == '2' ){ // band
        if( o[2] == 'C' ) return D2C;
        if( o[2] == 'D' ) return D2D;
        if( o[2] == 'I' ) return D2I;
        if( o[2] == 'L' ) return D2L;
        if( o[2] == 'M' ) return D2M;
        if( o[2] == 'N' ) return D2N;
        if( o[2] == 'P' ) return D2P;
        if( o[2] == 'Q' ) return D2Q;
        if( o[2] == 'S' ) return D2S;
        if( o[2] == 'W' ) return D2W;
        if( o[2] == 'X' ) return D2X;
        if( o[2] == 'Y' ) return D2Y;
      }
      if( o[1] == '5' ){ // band
        if( o[2] == 'I' ) return D5I;
        if( o[2] == 'Q' ) return D5Q;
        if( o[2] == 'X' ) return D5X;
      }
      if( o[1] == '6' ){ // band
        if( o[2] == 'A' ) return D6A;
        if( o[2] == 'B' ) return D6B;
        if( o[2] == 'C' ) return D6C;
        if( o[2] == 'I' ) return D6I;
        if( o[2] == 'Q' ) return D6Q;
        if( o[2] == 'X' ) return D6X;
        if( o[2] == 'Z' ) return D6Z;
      }
      if( o[1] == '7' ){ // band
        if( o[2] == 'I' ) return D7I;
        if( o[2] == 'Q' ) return D7Q;
        if( o[2] == 'X' ) return D7X;
      }
      if( o[1] == '8' ){ // band
        if( o[2] == 'I' ) return D8I;
        if( o[2] == 'Q' ) return D8Q;
        if( o[2] == 'X' ) return D8X;
      }
    }
     
    if( o[0] == 'S' ){ // signal-to-noise ratio
      if( o[1] == '1' ){ // band
        if( o[2] == 'A' ) return S1A;
        if( o[2] == 'B' ) return S1B;
        if( o[2] == 'C' ) return S1C;
        if( o[2] == 'L' ) return S1L;
        if( o[2] == 'M' ) return S1M;
        if( o[2] == 'N' ) return S1N;
        if( o[2] == 'P' ) return S1P;
        if( o[2] == 'S' ) return S1S;
        if( o[2] == 'W' ) return S1W;
        if( o[2] == 'X' ) return S1X;
        if( o[2] == 'Y' ) return S1Y;
        if( o[2] == 'Z' ) return S1Z;
      }
      if( o[1] == '2' ){ // band
        if( o[2] == 'C' ) return S2C;
        if( o[2] == 'D' ) return S2D;
        if( o[2] == 'I' ) return S2I;
        if( o[2] == 'L' ) return S2L;
        if( o[2] == 'M' ) return S2M;
        if( o[2] == 'N' ) return S2N;
        if( o[2] == 'P' ) return S2P;
        if( o[2] == 'Q' ) return S2Q;
        if( o[2] == 'S' ) return S2S;
        if( o[2] == 'W' ) return S2W;
        if( o[2] == 'X' ) return S2X;
        if( o[2] == 'Y' ) return S2Y;
      }
      if( o[1] == '5' ){ // band
        if( o[2] == 'I' ) return S5I;
        if( o[2] == 'Q' ) return S5Q;
        if( o[2] == 'X' ) return S5X;
      }
      if( o[1] == '6' ){ // band
        if( o[2] == 'A' ) return S6A;
        if( o[2] == 'B' ) return S6B;
        if( o[2] == 'C' ) return S6C;
        if( o[2] == 'I' ) return S6I;
        if( o[2] == 'Q' ) return S6Q;
        if( o[2] == 'X' ) return S6X;
        if( o[2] == 'Z' ) return S6Z;
      }
      if( o[1] == '7' ){ // band
        if( o[2] == 'I' ) return S7I;
        if( o[2] == 'Q' ) return S7Q;
        if( o[2] == 'X' ) return S7X;
      }
      if( o[1] == '8' ){ // band
        if( o[2] == 'I' ) return S8I;
        if( o[2] == 'Q' ) return S8Q;
        if( o[2] == 'X' ) return S8X;
      }
    }
  }
   
  if( len == 2 ){
    if( o[0] == 'C' ){ // code
      if( o[1] == '1' ) return C1;    // unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q                    
      if( o[1] == '2' ) return C2;    // unknown mode GPS P,Y                                     COMPASS E2 I/Q   
      if( o[1] == '5' ) return C5;    // unknown mode                      Galileo E5a I/Q                         
      if( o[1] == '6' ) return C6;    // unknown mode                      Galileo E6  I/Q        COMPASS E6 I/Q   
      if( o[1] == '7' ) return C7;    // unknown mode                      Galielo E5b I/Q        COMPASS E5b I/Q  
      if( o[1] == '8' ) return C8;    // unknown mode                      Galileo E5  I/Q                         
      if( o[1] == 'A' ) return CA;    // unknown mode GPS C/A  GLONASS SA                                          
      if( o[1] == 'B' ) return CB;    // unknown mode GPS C1C                                                      
      if( o[1] == 'C' ) return CC;    // unknown mode GPS C2C                                                      
      if( o[1] == 'D' ) return CD;    // unknown mode          GLONASS SA                                          
    }
    if( o[0] == 'P' ){ // code
      if( o[1] == '1' ) return P1;    // unknown mode GPS P,Y  GLONASS HA
      if( o[1] == '2' ) return P2;    // unknown mode GPS P,Y  GLONASS HA
      if( o[1] == '5' ) return P5;    // unknown mode GPS P,Y            
    }
     
    if( o[0] == 'L' ){ // phase
      if( o[1] == '1' ) return L1;    // unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q                    
      if( o[1] == '2' ) return L2;    // unknown mode GPS P,Y                                     COMPASS E2 I/Q   
      if( o[1] == '5' ) return L5;    // unknown mode                      Galileo E5a I/Q                         
      if( o[1] == '6' ) return L6;    // unknown mode                      Galileo E6  I/Q        COMPASS E6 I/Q   
      if( o[1] == '7' ) return L7;    // unknown mode                      Galielo E5b I/Q        COMPASS E5b I/Q  
      if( o[1] == '8' ) return L8;    // unknown mode                      Galileo E5  I/Q                         
      if( o[1] == 'A' ) return LA;    // unknown mode GPS C/A  GLONASS SA                                          
      if( o[1] == 'B' ) return LB;    // unknown mode GPS C1C                                                      
      if( o[1] == 'C' ) return LC;    // unknown mode GPS C2C                                                      
      if( o[1] == 'D' ) return LD;    // unknown mode          GLONASS SA                                          
    }

    if( o[0] == 'D' ){ // doppler
      if( o[1] == '1' ) return D1;    // unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q                    
      if( o[1] == '2' ) return D2;    // unknown mode GPS P,Y                                     COMPASS E2 I/Q   
      if( o[1] == '5' ) return D5;    // unknown mode                      Galileo E5a I/Q                         
      if( o[1] == '6' ) return D6;    // unknown mode                      Galileo E6  I/Q        COMPASS E6 I/Q   
      if( o[1] == '7' ) return D7;    // unknown mode                      Galielo E5b I/Q        COMPASS E5b I/Q  
      if( o[1] == '8' ) return D8;    // unknown mode                      Galileo E5  I/Q                         
      if( o[1] == 'A' ) return DA;    // unknown mode GPS C/A  GLONASS SA                                          
      if( o[1] == 'B' ) return DB;    // unknown mode GPS C1C                                                      
      if( o[1] == 'C' ) return DC;    // unknown mode GPS C2C                                                      
      if( o[1] == 'D' ) return DD;    // unknown mode          GLONASS SA                                          
    }

    if( o[0] == 'S' ){ // signal-to-noise ratio
      if( o[1] == '1' ) return S1;    // unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q                    
      if( o[1] == '2' ) return S2;    // unknown mode GPS P,Y                                     COMPASS E2 I/Q   
      if( o[1] == '5' ) return S5;    // unknown mode                      Galileo E5a I/Q                         
      if( o[1] == '6' ) return S6;    // unknown mode                      Galileo E6  I/Q        COMPASS E6 I/Q   
      if( o[1] == '7' ) return S7;    // unknown mode                      Galielo E5b I/Q        COMPASS E5b I/Q  
      if( o[1] == '8' ) return S8;    // unknown mode                      Galileo E5  I/Q                         
      if( o[1] == 'A' ) return SA;    // unknown mode GPS C/A  GLONASS SA                                          
      if( o[1] == 'B' ) return SB;    // unknown mode GPS C1C                                                      
      if( o[1] == 'C' ) return SC;    // unknown mode GPS C2C                                                      
      if( o[1] == 'D' ) return SD;    // unknown mode          GLONASS SA                                          
    }
  }
   
  cerr << "*** warning: not defined observation code: " + o << endl;
  // cannot be while static function !!!
  //  if( _log ) _log->comment(0,"gobsgnss","warning: not defined observation code: " + obs);
  //  else                      cerr << "*** warning: not defined observation code: " + obs << endl;
 
 return X;  // LAST_GOBS;
}


// convert GOBS to observation code string
// -----------
string t_gobs::gobs2str( const GOBS& o )
{
// PROBLEM WITH STATIC FUNCTION
//  boost::mutex::scoped_lock lock(_mutex);
   
  switch( o ){

    // CODE
    case C1A : return "C1A";  
    case C1B : return "C1B";  
    case C1C : return "C1C"; 
    case C1L : return "C1L";  
    case C1M : return "C1M";  
    case C1P : return "C1P"; 
    case C1S : return "C1S"; 
    case C1W : return "C1W";  
    case C1X : return "C1X";  
    case C1Y : return "C1Y";  
    case C1Z : return "C1Z";  
     
    case C2C : return "C2C";  
    case C2D : return "C2D";  
    case C2I : return "C2I";  
    case C2L : return "C2L";  
    case C2M : return "C2M";  
    case C2P : return "C2P";  
    case C2Q : return "C2Q";  
    case C2S : return "C2S";  
    case C2W : return "C2W";  
    case C2X : return "C2X";  
    case C2Y : return "C2Y";  
     
    case C5I : return "C5I";  
    case C5Q : return "C5Q";  
    case C5X : return "C5X";  
     
    case C6A : return "C6A";  
    case C6B : return "C6B";  
    case C6C : return "C6C";  
    case C6I : return "C6I";  
    case C6Q : return "C6Q"; 
    case C6X : return "C6X";  
    case C6Z : return "C6Z"; 
     
    case C7I : return "C7I";  
    case C7Q : return "C7Q"; 
    case C7X : return "C7X"; 
     
    case C8I : return "C8I";  
    case C8Q : return "C8Q";  
    case C8X : return "C8X"; 
     

    // CARRIER-PHASE
    case L1A : return "L1A";  
    case L1B : return "L1B";  
    case L1C : return "L1C";  
    case L1L : return "L1L";  
    case L1M : return "L1M";  
    case L1N : return "L1N";  
    case L1P : return "L1P";  
    case L1S : return "L1S";  
    case L1W : return "L1W";  
    case L1X : return "L1X";  
    case L1Y : return "L1Y";  
    case L1Z : return "L1Z"; 
     
    case L2C : return "L2C";  
    case L2D : return "L2D";  
    case L2I : return "L2I"; 
    case L2L : return "L2L";  
    case L2M : return "L2M";  
    case L2N : return "L2N"; 
    case L2P : return "L2P";  
    case L2Q : return "L2Q"; 
    case L2S : return "L2S";  
    case L2W : return "L2W"; 
    case L2X : return "L2X";  
    case L2Y : return "L2Y";  
     
    case L5I : return "L5I";  
    case L5Q : return "L5Q";  
    case L5X : return "L5X"; 

    case L6A : return "L6A";  
    case L6B : return "L6B";  
    case L6C : return "L6C";  
    case L6I : return "L6I";  
    case L6Q : return "L6Q";  
    case L6X : return "L6X";  
    case L6Z : return "L6Z";  

    case L7I : return "L7I";  
    case L7Q : return "L7Q";  
    case L7X : return "L7X";  
     
    case L8I : return "L8I";  
    case L8Q : return "L8Q";  
    case L8X : return "L8X";  

    // DOPLER
    case D1A : return "D1A";  
    case D1B : return "D1B";  
    case D1C : return "D1C";  
    case D1L : return "D1L";  
    case D1M : return "D1M";  
    case D1N : return "D1N";  
    case D1P : return "D1P";  
    case D1S : return "D1S";  
    case D1W : return "D1W";  
    case D1X : return "D1X";  
    case D1Y : return "D1Y";  
    case D1Z : return "D1Z"; 
     
    case D2C : return "D2C";  
    case D2D : return "D2D";  
    case D2I : return "D2I"; 
    case D2L : return "D2L";  
    case D2M : return "D2M";  
    case D2N : return "D2N";  
    case D2P : return "D2P";  
    case D2Q : return "D2Q";  
    case D2S : return "D2S";  
    case D2W : return "D2W";  
    case D2X : return "D2X";  
    case D2Y : return "D2Y"; 
     
    case D5I : return "D5I";  
    case D5Q : return "D5Q";  
    case D5X : return "D5X"; 
     
    case D6A : return "D6A";  
    case D6B : return "D6B";  
    case D6C : return "D6C";  
    case D6I : return "D6I";  
    case D6Q : return "D6Q"; 
    case D6X : return "D6X";  
    case D6Z : return "D6Z";  
     
    case D7I : return "D7I";  
    case D7Q : return "D7Q";  
    case D7X : return "D7X";  
     
    case D8I : return "D8I";  
    case D8Q : return "D8Q";  
    case D8X : return "D8X";  
       		      
    // SNR
    case S1A : return "S1A";  
    case S1B : return "S1B";  
    case S1C : return "S1C";  
    case S1L : return "S1L";  
    case S1M : return "S1M";  
    case S1N : return "S1N";  
    case S1P : return "S1P";  
    case S1S : return "S1S";  
    case S1W : return "S1W";  
    case S1X : return "S1X";  
    case S1Y : return "S1Y";  
    case S1Z : return "S1Z";  
     
    case S2C : return "S2C";  
    case S2D : return "S2D";  
    case S2I : return "S2I"; 
    case S2L : return "S2L";  
    case S2M : return "S2M";  
    case S2N : return "S2N";  
    case S2P : return "S2P";  
    case S2Q : return "S2Q";  
    case S2S : return "S2S";  
    case S2W : return "S2W";  
    case S2X : return "S2X";  
    case S2Y : return "S2Y";  
     
    case S5I : return "S5I";  
    case S5Q : return "S5Q";  
    case S5X : return "S5X";  
     
    case S6A : return "S6A"; 
    case S6B : return "S6B"; 
    case S6C : return "S6C"; 
    case S6I : return "S6I"; 
    case S6Q : return "S6Q"; 
    case S6X : return "S6X"; 
    case S6Z : return "S6Z"; 
          
    case S7I : return "S7I";  
    case S7Q : return "S7Q";  
    case S7X : return "S7X";  

    case S8I : return "S8I";  
    case S8Q : return "S8Q";  
    case S8X : return "S8X"; 
     
    // special cases: v2.x or unknown tracking mode
    case C1  : return "C1 ";    // unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q
    case C2  : return "C2 ";    // unknown mode GPS P,Y                                     COMPASS E2 I/Q
    case C5  : return "C5 ";    // unknown mode                      Galileo E5a I/Q
    case C6  : return "C6 ";    // unknown mode                      Galileo E6  I/Q        COMPASS E6 I/Q
    case C7  : return "C7 ";    // unknown mode                      Galielo E5b I/Q        COMPASS E5b I/Q
    case C8  : return "C8 ";    // unknown mode                      Galileo E5  I/Q
    case CA  : return "CA ";    // unknown mode GPS C/A  GLONASS SA
    case CB  : return "CB ";    // unknown mode GPS C1C
    case CC  : return "CC ";    // unknown mode GPS C2C
    case CD  : return "CD ";    // unknown mode          GLONASS SA
   	      	      
    case P1  : return "P1 ";    // unknown mode GPS P,Y  GLONASS HA
    case P2  : return "P2 ";    // unknown mode GPS P,Y  GLONASS HA
    case P5  : return "P5 ";    // unknown mode GPS P,Y
   	      	      
    case L1  : return "L1 ";    // unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q
    case L2  : return "L2 ";    // unknown mode GPS P,Y                                     COMPASS E2 I/Q
    case L5  : return "L5 ";    // unknown mode                      Galileo E5a I/Q
    case L6  : return "L6 ";    // unknown mode                      Galileo E6  I/Q        COMPASS E6 I/Q
    case L7  : return "L7 ";    // unknown mode                      Galielo E5b I/Q        COMPASS E5b I/Q
    case L8  : return "L8 ";    // unknown mode                      Galileo E5  I/Q
    case LA  : return "LA ";    // unknown mode GPS C/A  GLONASS SA
    case LB  : return "LB ";    // unknown mode GPS C1C
    case LC  : return "LC ";    // unknown mode GPS C2C
    case LD  : return "LD ";    // unknown mode          GLONASS SA
     	      	      
    case D1  : return "D1 ";    // unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q
    case D2  : return "D2 ";    // unknown mode GPS P,Y                                     COMPASS E2 I/Q
    case D5  : return "D5 ";    // unknown mode                      Galileo E5a I/Q
    case D6  : return "D6 ";    // unknown mode                      Galileo E6  I/Q        COMPASS E6 I/Q
    case D7  : return "D7 ";    // unknown mode                      Galielo E5b I/Q        COMPASS E5b I/Q
    case D8  : return "D8 ";    // unknown mode                      Galileo E5  I/Q
    case DA  : return "DA ";    // unknown mode GPS C/A  GLONASS SA
    case DB  : return "DB ";    // unknown mode GPS C1C
    case DC  : return "DC ";    // unknown mode GPS C2C
    case DD  : return "DD ";    // unknown mode          GLONASS SA
     	      	      
    case S1  : return "S1 ";    // unknown mode GPS P,Y              Galileo E2-L1-E2 I/Q
    case S2  : return "S2 ";    // unknown mode GPS P,Y                                     COMPASS E2 I/Q
    case S5  : return "S5 ";    // unknown mode                      Galileo E5a I/Q
    case S6  : return "S6 ";    // unknown mode                      Galileo E6  I/Q        COMPASS E6 I/Q
    case S7  : return "S7 ";    // unknown mode                      Galielo E5b I/Q        COMPASS E5b I/Q
    case S8  : return "S8 ";    // unknown mode                      Galileo E5  I/Q
    case SA  : return "SA ";    // unknown mode GPS C/A  GLONASS SA
    case SB  : return "SB ";    // unknown mode GPS C1C
    case SC  : return "SC ";    // unknown mode GPS C2C
    case SD  : return "SD ";    // unknown mode          GLONASS SA

//    case LAST_GOBS: return "XXX";
    case X : return "XXX";
              
    default : cerr << "*** warning: unknown observation code " << o << "\n";
              // cannot be while static function !!!
              // if( _log ) _log->comment(0,"gobsgnss","warning: unknown observation code ");
              // else                      cerr << "*** warning: unknown observation code \n"; 
  }

 return "XXX";
}

// convert phase to snr obstype
// -----------
GOBS t_gobs::pha2snr( const GOBS& o )
{
  return GOBS(o+200);
}
